//
// org.osid.Metadata
//
//     The OSID Java specification for org.osid.Metadata.
//
// Copyright (C) 2002-2004, 2006-2008 Massachusetts Institute of Technology. 
// 
// Copyright (C) 2009-2012 Ingenescus. All Rights Reserved. 
//
//     This Work is being provided by the copyright holder(s) subject to the 
//     following license. By obtaining, using and/or copying this Work, you 
//     agree that you have read, understand, and will comply with the 
//     following terms and conditions. 
//     
//     Permission to use, copy and distribute unmodified versions of this 
//     Work, for any purpose, without fee or royalty is hereby granted, 
//     provided that you include the above copyright notices and the terms of 
//     this license on ALL copies of the Work or portions thereof. 
//     
//     You may modify or create Derivatives of this Work only for your 
//     internal purposes. You shall not distribute or transfer any such 
//     Derivative of this Work to any location or to any third party. For the 
//     purposes of this license, "Derivative" shall mean any derivative of the 
//     Work as defined in the United States Copyright Act of 1976, such as a 
//     translation or modification. 
//     
//     This Work and the information contained herein is provided on an "AS 
//     IS" basis WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING 
//     BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
//     PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS 
//     OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
//     LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
//     FROM, OUT OF OR IN CONNECTION WITH THE WORK OR THE USE OR OTHER 
//     DEALINGS IN THE WORK. 
//     
//     The export of software employing encryption technology may require a 
//     specific license from the United States Government. It is the 
//     responsibility of any person or organization contemplating export to 
//     obtain such a license before exporting this Work. 
//

package org.osid;


/**
 *  <p>The <code> Metadata </code> interface defines a set of methods 
 *  describing a the syntax and rules for creating and updating a data element 
 *  inside an <code> OsidForm. </code> This interface provides a means to 
 *  retrieve special restrictions placed upon data elements such as sizes and 
 *  ranges that may vary from provider to provider or from object to object. 
 *  </p>
 */

public interface Metadata {


    /**
     *  Gets a unique <code> Id </code> for the data element. 
     *
     *  @return an <code> Id </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.id.Id getElementId();


    /**
     *  Gets a display label for the data element. 
     *
     *  @return a display label 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.locale.DisplayText getElementLabel();


    /**
     *  Gets instructions for updating this element value. This is a human 
     *  readable description of the data element or property that may include 
     *  special instructions or caveats to the end-user above and beyond what 
     *  this interface provides. 
     *
     *  @return instructions 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.locale.DisplayText getInstructions();


    /**
     *  Gets the syntax of this data. 
     *
     *  @return an enumeration indicating the <code> </code> type of value 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.Syntax getSyntax();


    /**
     *  Tests if this data element is an array. 
     *
     *  @return <code> true </code> if this data is an array, <code> false 
     *          </code> if a single element 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean isArray();


    /**
     *  Tests if this data element is required for creating new objects. 
     *
     *  @return <code> true </code> if this element value is required, <code> 
     *          false </code> otherwise 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean isRequired();


    /**
     *  Tests if this data can be updated. This may indicate the result of a 
     *  pre-authorization but is not a guarantee that an authorization failure 
     *  will not occur when the create or update transaction is issued. 
     *
     *  @return <code> true </code> if this data is not updatable, <code> 
     *          false </code> otherwise 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean isReadOnly();


    /**
     *  Tests if this data element is linked to other data in the object. 
     *  Updating linked data elements should refresh all metadata and 
     *  revalidate object elements. 
     *
     *  @return true if this element is linked, false if updates have no side 
     *          effect 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean isLinked();


    /**
     *  Tests if an existing value is known for this data element. If it is 
     *  known that a value does not exist, then this method returns <code> 
     *  true. </code> 
     *
     *  @return <code> true </code> if the element value is known, <code> 
     *          false </code> if the element value is not known 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean isValueKnown();


    /**
     *  Tests if this data element has a set non-default value. 
     *
     *  @return <code> true </code> if this element value has been set, <code> 
     *          false </code> otherwise 
     *  @throws org.osid.IllegalStateException <code> isValueKnown() </code> 
     *          is <code> false </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean hasValue();


    /**
     *  Gets the units of this data for display purposes ('lbs', 'gills', 
     *  'furlongs'). 
     *
     *  @return the display units of this data or an empty string if not 
     *          applicable 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.locale.DisplayText getUnits();


    /**
     *  In the case where an array or list of elements is specified in an 
     *  <code> OsidForm, </code> this specifies the minimum number of elements 
     *  that must be included. 
     *
     *  @return the minimum elements or <code> 1 </code> if <code> isArray() 
     *          </code> is <code> false </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public long getMinimumElements();


    /**
     *  In the case where an array or list of elements is specified in an 
     *  <code> OsidForm, </code> this specifies the maximum number of elements 
     *  that can be specified. 
     *
     *  @return the maximum elements or <code> 1 </code> if <code> isArray() 
     *          </code> is <code> false </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public long getMaximumElements();


    /**
     *  Gets the minimum cardinal value. 
     *
     *  @return the minimum cardinal 
     *  @throws org.osid.IllegalStateException syntax is not a <code> CARDINAL 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public long getMinimumCardinal();


    /**
     *  Gets the maximum cardinal value. 
     *
     *  @return the maximum cardinal 
     *  @throws org.osid.IllegalStateException syntax is not a <code> CARDINAL 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public long getMaximumCardinal();


    /**
     *  Gets the set of acceptable cardinal values. 
     *
     *  @return a set of cardinals or an empty array if not restricted 
     *  @throws org.osid.IllegalStateException syntax is not a <code> CARDINAL 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public long[] getCardinalSet();


    /**
     *  Gets the default cardinal values. These are the values used if the 
     *  element value is not provided or is cleared. If <code> isArray() 
     *  </code> is false, then this method returns at most a single value. 
     *
     *  @return the default cardinal values 
     *  @throws org.osid.IllegalStateException syntax is not a <code> CARDINAL 
     *          </code> or <code> isRequired() </code> is <code> true </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public long[] getDefaultCardinalValues();


    /**
     *  Gets the existing cardinal values. If <code> hasValue() </code> and 
     *  <code> isRequired() </code> are <code> false, </code> then these 
     *  values are the default values <code> . </code> If <code> isArray() 
     *  </code> is false, then this method returns at most a single value. 
     *
     *  @return the existing cardinal values 
     *  @throws org.osid.IllegalStateException syntax is not a <code> CARDINAL 
     *          </code> or <code> isValueKnown() </code> is <code> false 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public long[] getExistingCardinalValues();


    /**
     *  Gets the set of acceptable coordinate types. 
     *
     *  @return the set of coordinate types 
     *  @throws org.osid.IllegalStateException syntax is not a <code> 
     *          COORDINATE or SPATIALUNIT </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.type.Type[] getCoordinateTypes();


    /**
     *  Tests if the given coordinate type is supported. 
     *
     *  @param  coordinateType a coordinate Type 
     *  @return <code> true </code> if the type is supported, <code> false 
     *          </code> otherwise 
     *  @throws org.osid.IllegalStateException syntax is not a <code> 
     *          COORDINATE </code> 
     *  @throws org.osid.NullArgumentException <code> coordinateType </code> 
     *          is <code> null </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean supportsCoordinateType(org.osid.type.Type coordinateType);


    /**
     *  Gets the number of axes for a given supported coordinate type. 
     *
     *  @param  coordinateType a coordinate Type 
     *  @return the number of axes 
     *  @throws org.osid.IllegalStateException syntax is not a <code> 
     *          COORDINATE </code> 
     *  @throws org.osid.NullArgumentException <code> coordinateType </code> 
     *          is <code> null </code> 
     *  @throws org.osid.UnsupportedException <code> 
     *          supportsCoordinateType(coordinateType) </code> is <code> false 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public long getAxesForCoordinateType(org.osid.type.Type coordinateType);


    /**
     *  Gets the minimum coordinate values given supported coordinate type. 
     *
     *  @param  coordinateType a coordinate Type 
     *  @return the minimum coordinate values 
     *  @throws org.osid.IllegalStateException syntax is not a <code> 
     *          COORDINATE </code> 
     *  @throws org.osid.NullArgumentException <code> coordinateType </code> 
     *          is <code> null </code> 
     *  @throws org.osid.UnsupportedException <code> 
     *          supportsCoordinateType(coordinateType) </code> is <code> false 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public java.math.BigDecimal[] getMinimumCoordinateValues(org.osid.type.Type coordinateType);


    /**
     *  Gets the maximum coordinate values given supported coordinate type. 
     *
     *  @param  coordinateType a coordinate Type 
     *  @return the maximum coordinate values 
     *  @throws org.osid.IllegalStateException syntax is not a <code> 
     *          COORDINATE </code> 
     *  @throws org.osid.NullArgumentException <code> coordinateType </code> 
     *          is <code> null </code> 
     *  @throws org.osid.UnsupportedException <code> 
     *          supportsCoordinateType(coordinateType) </code> is <code> false 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public java.math.BigDecimal[] getMaximumCoordinateValues(org.osid.type.Type coordinateType);


    /**
     *  Gets the set of acceptable coordinate values. 
     *
     *  @return a set of coordinates or an empty array if not restricted 
     *  @throws org.osid.IllegalStateException syntax is not a <code> 
     *          COORDINATE </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.mapping.Coordinate[] getCoordinateSet();


    /**
     *  Gets the default coordinate values. These are the values used if the 
     *  element value is not provided or is cleared. If <code> isArray() 
     *  </code> is false, then this method returns at most a single value. 
     *
     *  @return the default coordinate values 
     *  @throws org.osid.IllegalStateException syntax is not a <code> 
     *          COORDINATE </code> or <code> isRequired() </code> is <code> 
     *          true </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.mapping.Coordinate[] getDefaultCoordinateValues();


    /**
     *  Gets the existing coordinate values. If <code> hasValue() </code> and 
     *  <code> isRequired() </code> are <code> false, </code> then these 
     *  values are the default values <code> . </code> If <code> isArray() 
     *  </code> is false, then this method returns at most a single value. 
     *
     *  @return the existing coordinate values 
     *  @throws org.osid.IllegalStateException syntax is not a <code> 
     *          COORDINATE </code> or <code> isValueKnown() </code> is <code> 
     *          false </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.mapping.Coordinate[] getExistingCoordinateValues();


    /**
     *  Gets the set of acceptable currency types. 
     *
     *  @return the set of currency types 
     *  @throws org.osid.IllegalStateException syntax is not a <code> CURRENCY 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.type.Type[] getCurrencyTypes();


    /**
     *  Tests if the given currency type is supported. 
     *
     *  @param  currencyType a currency Type 
     *  @return <code> true </code> if the type is supported, <code> false 
     *          </code> otherwise 
     *  @throws org.osid.IllegalStateException syntax is not a <code> CURRENCY 
     *          </code> 
     *  @throws org.osid.NullArgumentException <code> currencyType </code> is 
     *          <code> null </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean supportsCurrencyType(org.osid.type.Type currencyType);


    /**
     *  Gets the minimum currency value. 
     *
     *  @return the minimum currency 
     *  @throws org.osid.IllegalStateException syntax is not a <code> CURRENCY 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.financials.Currency getMinimumCurrency();


    /**
     *  Gets the maximum currency value. 
     *
     *  @return the maximum currency 
     *  @throws org.osid.IllegalStateException syntax is not a <code> CURRENCY 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.financials.Currency getMaximumCurrency();


    /**
     *  Gets the set of acceptable currency values. 
     *
     *  @return a set of currencies or an empty array if not restricted 
     *  @throws org.osid.IllegalStateException syntax is not a <code> CURRENCY 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.financials.Currency[] getCurrencySet();


    /**
     *  Gets the default currency values. These are the values used if the 
     *  element value is not provided or is cleared. If <code> isArray() 
     *  </code> is false, then this method returns at most a single value. 
     *
     *  @return the default currency values 
     *  @throws org.osid.IllegalStateException syntax is not a <code> CURRENCY 
     *          </code> or <code> isRequired() </code> is <code> true </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.financials.Currency[] getDefaultCurrencyValues();


    /**
     *  Gets the existing currency values. If <code> hasValue() </code> and 
     *  <code> isRequired() </code> are <code> false, </code> then these 
     *  values are the default values <code> . </code> If <code> isArray() 
     *  </code> is false, then this method returns at most a single value. 
     *
     *  @return the existing currency values 
     *  @throws org.osid.IllegalStateException syntax is not a <code> CURRENCY 
     *          </code> or <code> isValueKnown() </code> is <code> false 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.financials.Currency[] getExistingCurrencyValues();


    /**
     *  Gets the smallest resolution of the date time value. 
     *
     *  @return the resolution 
     *  @throws org.osid.IllegalStateException syntax is not a <code> 
     *          DATETIME, DURATION </code> , or <code> TIME </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.calendaring.DateTimeResolution getDateTimeResolution();


    /**
     *  Gets the set of acceptable calendar types. 
     *
     *  @return the set of calendar types 
     *  @throws org.osid.IllegalStateException syntax is not a <code> DATETIME 
     *          </code> or <code> DURATION </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.type.Type[] getCalendarTypes();


    /**
     *  Tests if the given calendar type is supported. 
     *
     *  @param  calendarType a calendar Type 
     *  @return <code> true </code> if the type is supported, <code> false 
     *          </code> otherwise 
     *  @throws org.osid.IllegalStateException syntax is not a <code> DATETIME 
     *          </code> or <code> DURATION </code> 
     *  @throws org.osid.NullArgumentException <code> calendarType </code> is 
     *          <code> null </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean supportsCalendarType(org.osid.type.Type calendarType);


    /**
     *  Gets the set of acceptable time types. 
     *
     *  @return a set of time types or an empty array if not restricted 
     *  @throws org.osid.IllegalStateException syntax is not a <code> 
     *          DATETIME, DURATION, </code> or <code> TIME </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.type.Type[] getTimeTypes();


    /**
     *  Tests if the given time type is supported. 
     *
     *  @param  timeType a time Type 
     *  @return <code> true </code> if the type is supported, <code> false 
     *          </code> otherwise 
     *  @throws org.osid.IllegalStateException syntax is not a <code> 
     *          DATETIME, DURATION, </code> or <code> TIME </code> 
     *  @throws org.osid.NullArgumentException <code> timeType </code> is 
     *          <code> null </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean supportsTimeType(org.osid.type.Type timeType);


    /**
     *  Gets the minimum date time value. 
     *
     *  @return the minimum value 
     *  @throws org.osid.IllegalStateException syntax is not a <code> DATETIME 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.calendaring.DateTime getMinimumDateTime();


    /**
     *  Gets the maximum date time value. 
     *
     *  @return the maximum value 
     *  @throws org.osid.IllegalStateException syntax is not a <code> DATETIME 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.calendaring.DateTime getMaximumDateTime();


    /**
     *  Gets the set of acceptable date time values. 
     *
     *  @return a set of values or an empty array if not restricted 
     *  @throws org.osid.IllegalStateException syntax is not a <code> DATETIME 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.calendaring.DateTime[] getDateTimeSet();


    /**
     *  Gets the default date time values. These are the values used if the 
     *  element value is not provided or is cleared. If <code> isArray() 
     *  </code> is false, then this method returns at most a single value. 
     *
     *  @return the default date time values 
     *  @throws org.osid.IllegalStateException syntax is not a <code> DATETIME 
     *          </code> or <code> isRequired() </code> is <code> true </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.calendaring.DateTime[] getDefaultDateTimeValues();


    /**
     *  Gets the existing date time values. If <code> hasValue() </code> and 
     *  <code> isRequired() </code> are <code> false, </code> then these 
     *  values are the default values <code> . </code> If <code> isArray() 
     *  </code> is false, then this method returns at most a single value. 
     *
     *  @return the existing date time values 
     *  @throws org.osid.IllegalStateException syntax is not a <code> DATETIME 
     *          </code> or <code> isValueKnown() </code> is <code> false 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.calendaring.DateTime[] getExistingDateTimeValues();


    /**
     *  Gets the number of digits to the right of the decimal point. 
     *
     *  @return the scale 
     *  @throws org.osid.IllegalStateException syntax is not a <code> DECIMAL 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public long getDecimalScale();


    /**
     *  Gets the minimum decimal value. 
     *
     *  @return the minimum decimal 
     *  @throws org.osid.IllegalStateException syntax is not a <code> DECIMAL 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public java.math.BigDecimal getMinimumDecimal();


    /**
     *  Gets the maximum decimal value. 
     *
     *  @return the maximum decimal 
     *  @throws org.osid.IllegalStateException syntax is not a <code> DECIMAL 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public java.math.BigDecimal getMaximumDecimal();


    /**
     *  Gets the set of acceptable decimal values. 
     *
     *  @return a set of decimals or an empty array if not restricted 
     *  @throws org.osid.IllegalStateException syntax is not a <code> DECIMAL 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public java.math.BigDecimal[] getDecimalSet();


    /**
     *  Gets the default decimal values. These are the values used if the 
     *  element value is not provided or is cleared. If <code> isArray() 
     *  </code> is false, then this method returns at most a single value. 
     *
     *  @return the default decimal values 
     *  @throws org.osid.IllegalStateException syntax is not a <code> DECIMAL 
     *          </code> or <code> isRequired() </code> is <code> true </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public java.math.BigDecimal[] getDefaultDecimalValues();


    /**
     *  Gets the existing decimal values. If <code> hasValue() </code> and 
     *  <code> isRequired() </code> are <code> false, </code> then these 
     *  values are the default values <code> . </code> If <code> isArray() 
     *  </code> is false, then this method returns at most a single value. 
     *
     *  @return the existing decimal values 
     *  @throws org.osid.IllegalStateException syntax is not a <code> DECIMAL 
     *          </code> or <code> isValueKnown() </code> is <code> false 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public java.math.BigDecimal[] getExistingDecimalValues();


    /**
     *  Gets the smallest resolution of the distance value. 
     *
     *  @return the resolution 
     *  @throws org.osid.IllegalStateException syntax is not a <code> DISTANCE 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.mapping.DistanceResolution getDistanceResolution();


    /**
     *  Gets the minimum distance value. 
     *
     *  @return the minimum value 
     *  @throws org.osid.IllegalStateException syntax is not a <code> DISTANCE 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.mapping.Distance getMinimumDistance();


    /**
     *  Gets the maximum distance value. 
     *
     *  @return the maximum value 
     *  @throws org.osid.IllegalStateException syntax is not a <code> DISTANCE 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.mapping.Distance getMaximumDistance();


    /**
     *  Gets the set of acceptable distance values. 
     *
     *  @return a set of values or an empty array if not restricted 
     *  @throws org.osid.IllegalStateException syntax is not a <code> DISTANCE 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.mapping.Distance[] getDistanceSet();


    /**
     *  Gets the default distance values. These are the values used if the 
     *  element value is not provided or is cleared. If <code> isArray() 
     *  </code> is false, then this method returns at most a single value. 
     *
     *  @return the default distance values 
     *  @throws org.osid.IllegalStateException syntax is not a <code> DISTANCE 
     *          </code> or <code> isRequired() </code> is <code> true </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.mapping.Distance[] getDefaultDistanceValues();


    /**
     *  Gets the existing distance values. If <code> hasValue() </code> and 
     *  <code> isRequired() </code> are <code> false, </code> then these 
     *  values are the default values <code> . </code> If <code> isArray() 
     *  </code> is false, then this method returns at most a single value. 
     *
     *  @return the existing distance values 
     *  @throws org.osid.IllegalStateException syntax is not a <code> DISTANCE 
     *          </code> or <code> isValueKnown() </code> is <code> false 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.mapping.Distance[] getExistingDistanceValues();


    /**
     *  Gets the minimum duration. 
     *
     *  @return the minimum duration 
     *  @throws org.osid.IllegalStateException syntax is not a <code> DURATION 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.calendaring.Duration getMinimumDuration();


    /**
     *  Gets the maximum duration. 
     *
     *  @return the maximum duration 
     *  @throws org.osid.IllegalStateException syntax is not a <code> DURATION 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.calendaring.Duration getMaximumDuration();


    /**
     *  Gets the set of acceptable duration values. 
     *
     *  @return a set of durations or an empty array if not restricted 
     *  @throws org.osid.IllegalStateException syntax is not a <code> DURATION 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.calendaring.Duration[] getDurationSet();


    /**
     *  Gets the default duration values. These are the values used if the 
     *  element value is not provided or is cleared. If <code> isArray() 
     *  </code> is false, then this method returns at most at most a single 
     *  value. 
     *
     *  @return the default duration values 
     *  @throws org.osid.IllegalStateException syntax is not a DURATION or 
     *          <code> isRequired() </code> is <code> true </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.calendaring.Duration[] getDefaultDurationValues();


    /**
     *  Gets the existing duration values. If <code> hasValue() </code> and 
     *  <code> isRequired() </code> are <code> false, </code> then these 
     *  values are the default values <code> . </code> If <code> isArray() 
     *  </code> is false, then this method returns at most a single value. 
     *
     *  @return the existing duration values 
     *  @throws org.osid.IllegalStateException syntax is not a <code> DURATION 
     *          </code> or <code> isValueKnown() </code> is <code> false 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.calendaring.Duration[] getExistingDurationValues();


    /**
     *  Gets the set of acceptable heading types. 
     *
     *  @return a set of heading types or an empty array if not restricted 
     *  @throws org.osid.IllegalStateException syntax is not a <code> HEADING 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.type.Type[] getHeadingTypes();


    /**
     *  Tests if the given heading type is supported. 
     *
     *  @param  headingType a heading Type 
     *  @return <code> true </code> if the type is supported, <code> false 
     *          </code> otherwise 
     *  @throws org.osid.IllegalStateException syntax is not a <code> HEADING 
     *          </code> 
     *  @throws org.osid.NullArgumentException <code> headingType </code> is 
     *          <code> null </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean supportsHeadingType(org.osid.type.Type headingType);


    /**
     *  Gets the number of axes for a given supported heading type. 
     *
     *  @param  headingType a heading Type 
     *  @return the number of axes 
     *  @throws org.osid.IllegalStateException syntax is not a <code> HEADING 
     *          </code> 
     *  @throws org.osid.NullArgumentException <code> headingType </code> is 
     *          <code> null </code> 
     *  @throws org.osid.UnsupportedException <code> 
     *          supportsHeadingType(headingType) </code> is <code> false 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public long getAxesForHeadingType(org.osid.type.Type headingType);


    /**
     *  Gets the minimum heading values given supported heading type. 
     *
     *  @param  headingType a heading Type 
     *  @return the minimum heading values 
     *  @throws org.osid.IllegalStateException syntax is not a <code> HEADING 
     *          </code> 
     *  @throws org.osid.NullArgumentException <code> headingType </code> is 
     *          <code> null </code> 
     *  @throws org.osid.UnsupportedException <code> 
     *          supportsHeadingType(headingType) </code> is <code> false 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public java.math.BigDecimal[] getMinimumHeadingValues(org.osid.type.Type headingType);


    /**
     *  Gets the maximum heading values given supported heading type. 
     *
     *  @param  headingType a heading Type 
     *  @return the maximum heading values 
     *  @throws org.osid.IllegalStateException syntax is not a <code> HEADING 
     *          </code> 
     *  @throws org.osid.NullArgumentException <code> headingType </code> is 
     *          <code> null </code> 
     *  @throws org.osid.UnsupportedException <code> 
     *          supportsHeadingType(headingType) </code> is <code> false 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public java.math.BigDecimal[] getMaximumHeadingValues(org.osid.type.Type headingType);


    /**
     *  Gets the set of acceptable heading values. 
     *
     *  @return the set of heading 
     *  @throws org.osid.IllegalStateException syntax is not a <code> HEADING 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.mapping.Heading[] getHeadingSet();


    /**
     *  Gets the default heading values. These are the values used if the 
     *  element value is not provided or is cleared. If <code> isArray() 
     *  </code> is false, then this method returns at most a single value. 
     *
     *  @return the default heading values 
     *  @throws org.osid.IllegalStateException syntax is not a <code> HEADING 
     *          </code> or <code> isRequired() </code> is <code> true </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.mapping.Heading[] getDefaultHeadingValues();


    /**
     *  Gets the existing heading values. If <code> hasValue() </code> and 
     *  <code> isRequired() </code> are <code> false, </code> then these 
     *  values are the default values <code> . </code> If <code> isArray() 
     *  </code> is false, then this method returns at most a single value. 
     *
     *  @return the existing heading values 
     *  @throws org.osid.IllegalStateException syntax is not a <code> HEADING 
     *          </code> or <code> isValueKnown() </code> is <code> false 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.mapping.Heading[] getExistingHeadingValues();


    /**
     *  Gets the set of acceptable <code> Ids. </code> 
     *
     *  @return a set of <code> Ids </code> or an empty array if not 
     *          restricted 
     *  @throws org.osid.IllegalStateException syntax is not an <code> ID 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.id.Id[] getIdSet();


    /**
     *  Gets the default <code> Id </code> values. These are the values used 
     *  if the element value is not provided or is cleared. If <code> 
     *  isArray() </code> is false, then this method returns at most a single 
     *  value. 
     *
     *  @return the default <code> Id </code> values 
     *  @throws org.osid.IllegalStateException syntax is not an <code> ID 
     *          </code> or <code> isRequired() </code> is <code> true </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.id.Id[] getDefaultIdValues();


    /**
     *  Gets the existing <code> Id </code> values. If <code> hasValue() 
     *  </code> and <code> isRequired() </code> are <code> false, </code> then 
     *  these values are the default values <code> . </code> If <code> 
     *  isArray() </code> is false, then this method returns at most a single 
     *  value. 
     *
     *  @return the existing <code> Id </code> values 
     *  @throws org.osid.IllegalStateException syntax is not an <code> ID 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.id.Id[] getExistingIdValues();


    /**
     *  Gets the minimum integer value. 
     *
     *  @return the minimum value 
     *  @throws org.osid.IllegalStateException syntax is not an <code> INTEGER 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public long getMinimumInteger();


    /**
     *  Gets the maximum integer value. 
     *
     *  @return the maximum value 
     *  @throws org.osid.IllegalStateException syntax is not an <code> INTEGER 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public long getMaximumInteger();


    /**
     *  Gets the set of acceptable integer values. 
     *
     *  @return a set of values or an empty array if not restricted 
     *  @throws org.osid.IllegalStateException syntax is not an <code> INTEGER 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public long[] getIntegerSet();


    /**
     *  Gets the default integer values. These are the values used if the 
     *  element value is not provided or is cleared. If <code> isArray() 
     *  </code> is false, then this method returns at most a single value. 
     *
     *  @return the default integer values 
     *  @throws org.osid.IllegalStateException syntax is not an <code> INTEGER 
     *          </code> or <code> isRequired() </code> is <code> true </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public long[] getDefaultIntegerValues();


    /**
     *  Gets the existing integer values. If <code> hasValue() </code> and 
     *  <code> isRequired() </code> are <code> false, </code> then these 
     *  values are the default values <code> . </code> If <code> isArray() 
     *  </code> is false, then this method returns at most a single value. 
     *
     *  @return the existing integer values 
     *  @throws org.osid.IllegalStateException syntax is not a <code> INTEGER 
     *          </code> or isValueKnown() is false 
     *  @compliance mandatory This method must be implemented. 
     */

    public long[] getExistingIntegerValues();


    /**
     *  Gets the set of acceptable <code> Types </code> for an arbitrary 
     *  object. 
     *
     *  @return a set of <code> Types </code> or an empty array if not 
     *          restricted 
     *  @throws org.osid.IllegalStateException syntax is not an <code> OBJECT 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.type.Type[] getObjectTypes();


    /**
     *  Tests if the given object type is supported. 
     *
     *  @param  objectType an object Type 
     *  @return <code> true </code> if the type is supported, <code> false 
     *          </code> otherwise 
     *  @throws org.osid.IllegalStateException syntax is not an <code> OBJECT 
     *          </code> 
     *  @throws org.osid.NullArgumentException <code> objectType </code> is 
     *          <code> null </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean supportsObjectType(org.osid.type.Type objectType);


    /**
     *  Gets the set of acceptable object values. 
     *
     *  @return a set of values or an empty array if not restricted 
     *  @throws org.osid.IllegalStateException syntax is not an <code> OBJECT 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public java.lang.Object[] getObjectSet();


    /**
     *  Gets the default object values. These are the values used if the 
     *  element value is not provided or is cleared. If <code> isArray() 
     *  </code> is false, then this method returns at most a single value. 
     *
     *  @return the default object values 
     *  @throws org.osid.IllegalStateException syntax is not an <code> OBJECT 
     *          </code> or <code> isRequired() </code> is <code> true </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public java.lang.Object[] getDefaultObjectValues();


    /**
     *  Gets the existing object values. If <code> hasValue() </code> and 
     *  <code> isRequired() </code> are <code> false, </code> then these 
     *  values are the default values <code> . </code> If <code> isArray() 
     *  </code> is false, then this method returns at most a single value. 
     *
     *  @return the existing object values 
     *  @throws org.osid.IllegalStateException syntax is not an OBJECT or 
     *          <code> isValueKnown() </code> is <code> false </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public java.lang.Object[] getExistingObjectValues();


    /**
     *  Gets the set of acceptable spatial unit record types. 
     *
     *  @return the set of spatial unit types 
     *  @throws org.osid.IllegalStateException syntax is not <code> 
     *          SPATIALUNIT </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.type.Type[] getSpatialUnitRecordTypes();


    /**
     *  Tests if the given spatial unit record type is supported. 
     *
     *  @param  spatialUnitRecordType a spatial unit record Type 
     *  @return <code> true </code> if the type is supported, <code> false 
     *          </code> otherwise 
     *  @throws org.osid.IllegalStateException syntax is not an <code> 
     *          SPATIALUNIT </code> 
     *  @throws org.osid.NullArgumentException <code> spatialUnitRecordType 
     *          </code> is <code> null </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean supportsSpatialUnitRecordType(org.osid.type.Type spatialUnitRecordType);


    /**
     *  Gets the set of acceptable spatial unit values. 
     *
     *  @return a set of spatial units or an empty array if not restricted 
     *  @throws org.osid.IllegalStateException syntax is not a <code> 
     *          SPATIALUNIT </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.mapping.SpatialUnit[] getSpatialUnitSet();


    /**
     *  Gets the default spatial unit values. These are the values used if the 
     *  element value is not provided or is cleared. If <code> isArray() 
     *  </code> is false, then this method returns at most a single value. 
     *
     *  @return the default spatial unit values 
     *  @throws org.osid.IllegalStateException syntax is not a <code> 
     *          SPATIALUNIT </code> or <code> isRequired() </code> is <code> 
     *          true </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.mapping.SpatialUnit[] getDefaultSpatialUnitValues();


    /**
     *  Gets the existing spatial unit values. If <code> hasValue() </code> 
     *  and <code> isRequired() </code> are <code> false, </code> then these 
     *  values are the default values <code> . </code> If <code> isArray() 
     *  </code> is false, then this method returns at most a single value. 
     *
     *  @return the existing spatial unit values 
     *  @throws org.osid.IllegalStateException syntax is not a SPATIALUNIT or 
     *          <code> isValueKnown() </code> is <code> false </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.mapping.SpatialUnit[] getExistingSpatialUnitValues();


    /**
     *  Gets the minimum speed value. 
     *
     *  @return the minimum speed 
     *  @throws org.osid.IllegalStateException syntax is not a <code> SPEED 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.mapping.Speed getMinimumSpeed();


    /**
     *  Gets the maximum speed value. 
     *
     *  @return the maximum speed 
     *  @throws org.osid.IllegalStateException syntax is not a <code> SPEED 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.mapping.Speed getMaximumSpeed();


    /**
     *  Gets the set of acceptable speed values. 
     *
     *  @return a set of speeds or an empty array if not restricted 
     *  @throws org.osid.IllegalStateException syntax is not a <code> SPEED 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.mapping.Speed[] getSpeedSet();


    /**
     *  Gets the default speed values. These are the values used if the 
     *  element value is not provided or is cleared. If <code> isArray() 
     *  </code> is false, then this method returns at most a single value. 
     *
     *  @return the default speed values 
     *  @throws org.osid.IllegalStateException syntax is not a <code> SPEED 
     *          </code> or <code> isRequired() </code> is <code> true </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.mapping.Speed[] getDefaultSpeedValues();


    /**
     *  Gets the existing speed values. If <code> hasValue() </code> and 
     *  <code> isRequired() </code> are <code> false, </code> then these 
     *  values are the default values <code> . </code> If <code> isArray() 
     *  </code> is false, then this method returns at most a single value. 
     *
     *  @return the existing speed values 
     *  @throws org.osid.IllegalStateException syntax is not a <code> SPEED 
     *          </code> or <code> isValueKnown() </code> is <code> false 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.mapping.Speed[] getExistingSpeedValues();


    /**
     *  Gets the minimum string length. 
     *
     *  @return the minimum string length 
     *  @throws org.osid.IllegalStateException syntax is not a <code> STRING 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public long getMinimumStringLength();


    /**
     *  Gets the maximum string length. 
     *
     *  @return the maximum string length 
     *  @throws org.osid.IllegalStateException syntax is not a <code> STRING 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public long getMaximumStringLength();


    /**
     *  Gets the set of valid string match types for use in validating a 
     *  string. If the string match type indicates a regular expression then 
     *  <code> getStringExpression() </code> returns a regular expression. 
     *
     *  @return the set of string match types 
     *  @throws org.osid.IllegalStateException syntax is not a <code> STRING 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.type.Type[] getStringMatchTypes();


    /**
     *  Tests if the given string match type is supported. 
     *
     *  @param  stringMatchType a string match type 
     *  @return <code> true </code> if the given string match type Is 
     *          supported, <code> false </code> otherwise 
     *  @throws org.osid.IllegalStateException syntax is not a <code> STRING 
     *          </code> 
     *  @throws org.osid.NullArgumentException <code> stringMatchType </code> 
     *          is <code> null </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean supportsStringMatchType(org.osid.type.Type stringMatchType);


    /**
     *  Gets the regular expression of an acceptable string for the given 
     *  string match type. 
     *
     *  @param  stringMatchType a string match type 
     *  @return the regular expression 
     *  @throws org.osid.NullArgumentException <code> stringMatchType </code> 
     *          is <code> null </code> 
     *  @throws org.osid.IllegalStateException syntax is not a <code> STRING 
     *          </code> 
     *  @throws org.osid.UnsupportedException <code> 
     *          supportsStringMatchType(stringMatchType </code> ) is <code> 
     *          false </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public String getStringExpression(org.osid.type.Type stringMatchType);


    /**
     *  Gets the set of valid string formats. 
     *
     *  @return the set of valid text format types 
     *  @throws org.osid.IllegalStateException syntax is not a <code> STRING 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.type.Type[] getStringFormatTypes();


    /**
     *  Gets the set of acceptable string values. 
     *
     *  @return a set of strings or an empty array if not restricted 
     *  @throws org.osid.IllegalStateException syntax is not a <code> STRING 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public String[] getStringSet();


    /**
     *  Gets the default string values. These are the values used if the 
     *  element value is not provided or is cleared. If <code> isArray() 
     *  </code> is false, then this method returns at most a single value. 
     *
     *  @return the default string values 
     *  @throws org.osid.IllegalStateException syntax is not a <code> STRING 
     *          </code> or <code> isRequired() </code> is <code> true </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public String[] getDefaultStringValues();


    /**
     *  Gets the existing string values. If <code> hasValue() </code> and 
     *  <code> isRequired() </code> are <code> false, </code> then these 
     *  values are the default values <code> . </code> If <code> isArray() 
     *  </code> is false, then this method returns at most a single value. 
     *
     *  @return the existing string values 
     *  @throws org.osid.IllegalStateException syntax is not a <code> STRING 
     *          </code> or <code> isValueKnown() </code> is <code> false 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public String[] getExistingStringValues();


    /**
     *  Gets the minimum time value. 
     *
     *  @return the minimum time 
     *  @throws org.osid.IllegalStateException syntax is not a <code> TIME 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.calendaring.Time getMinimumTime();


    /**
     *  Gets the maximum time value. 
     *
     *  @return the maximum time 
     *  @throws org.osid.IllegalStateException syntax is not a <code> TIME 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.calendaring.Time getMaximumTime();


    /**
     *  Gets the set of acceptable time values. 
     *
     *  @return a set of times or an empty array if not restricted 
     *  @throws org.osid.IllegalStateException syntax is not a <code> TIME 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.calendaring.Time[] getTimeSet();


    /**
     *  Gets the default time values. These are the values used if the element 
     *  value is not provided or is cleared. If <code> isArray() </code> is 
     *  false, then this method returns at most a single value. 
     *
     *  @return the default time values 
     *  @throws org.osid.IllegalStateException syntax is not a <code> TIME 
     *          </code> or <code> isRequired() </code> is <code> true </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.calendaring.Time[] getDefaultTimeValues();


    /**
     *  Gets the existing time values. If <code> hasValue() </code> and <code> 
     *  isRequired() </code> are <code> false, </code> then these values are 
     *  the default values <code> . </code> If <code> isArray() </code> is 
     *  false, then this method returns at most a single value. 
     *
     *  @return the existing time values 
     *  @throws org.osid.IllegalStateException syntax is not a <code> TIME 
     *          </code> or <code> isValueKnown() </code> is <code> false 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.calendaring.Time[] getExistingTimeValues();


    /**
     *  Gets the set of acceptable <code> Types. </code> 
     *
     *  @return a set of <code> Types </code> or an empty array if not 
     *          restricted 
     *  @throws org.osid.IllegalStateException syntax is not a <code> TYPE 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.type.Type[] getTypeSet();


    /**
     *  Gets the default type values. These are the values used if the element 
     *  value is not provided or is cleared. If <code> isArray() </code> is 
     *  false, then this method returns at most a single value. 
     *
     *  @return the default type values 
     *  @throws org.osid.IllegalStateException syntax is not a <code> TYPE 
     *          </code> or <code> isRequired() </code> is <code> true </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.type.Type[] getDefaultTypeValues();


    /**
     *  Gets the existing type values. If <code> hasValue() </code> and <code> 
     *  isRequired() </code> are <code> false, </code> then these values are 
     *  the default values <code> . </code> If <code> isArray() </code> is 
     *  false, then this method returns at most a single value. 
     *
     *  @return the existing type values 
     *  @throws org.osid.IllegalStateException syntax is not a <code> TYPE 
     *          </code> or <code> isValueKnown() </code> is <code> false 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.type.Type[] getExistingTypeValues();


    /**
     *  Gets the set of acceptable version types. 
     *
     *  @return the set of version types 
     *  @throws org.osid.IllegalStateException syntax is not a <code> VERSION 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.type.Type[] getVersionTypes();


    /**
     *  Tests if the given version type is supported. 
     *
     *  @param  versionType a version Type 
     *  @return <code> true </code> if the type is supported, <code> false 
     *          </code> otherwise 
     *  @throws org.osid.IllegalStateException syntax is not a <code> VERSION 
     *          </code> 
     *  @throws org.osid.NullArgumentException <code> versionType </code> is 
     *          <code> null </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean supportsVersionType(org.osid.type.Type versionType);


    /**
     *  Gets the minumim acceptable <code> Version. </code> 
     *
     *  @return the minumim <code> Version </code> 
     *  @throws org.osid.IllegalStateException syntax is not a <code> VERSION 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.installation.Version getMinimumVersion();


    /**
     *  Gets the maximum acceptable <code> Version. </code> 
     *
     *  @return the maximum <code> Version </code> 
     *  @throws org.osid.IllegalStateException syntax is not a <code> VERSION 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.installation.Version getMaximumVersion();


    /**
     *  Gets the set of acceptable <code> Versions. </code> 
     *
     *  @return a set of <code> Versions </code> or an empty array if not 
     *          restricted 
     *  @throws org.osid.IllegalStateException syntax is not a <code> VERSION 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.installation.Version[] getVersionSet();


    /**
     *  Gets the default version values. These are the values used if the 
     *  element value is not provided or is cleared. If <code> isArray() 
     *  </code> is false, then this method returns at most a single value. 
     *
     *  @return the default version values 
     *  @throws org.osid.IllegalStateException syntax is not a TIME or 
     *          isValueKnown() is false 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.installation.Version[] getDefaultVersionValues();


    /**
     *  Gets the existing version values. If <code> hasValue() </code> and 
     *  <code> isRequired() </code> are <code> false, </code> then these 
     *  values are the default values <code> . </code> If <code> isArray() 
     *  </code> is false, then this method returns at most a single value. 
     *
     *  @return the existing version values 
     *  @throws org.osid.IllegalStateException syntax is not a <code> VERSION 
     *          </code> or <code> isValueKnown() </code> is <code> false 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.installation.Version[] getExistingVersionValues();
}
