//
// org.osid.OsidForm
//
//     The OSID Java specification for org.osid.OsidForm.
//
// Copyright (C) 2002-2004, 2006-2008 Massachusetts Institute of Technology. 
// 
// Copyright (C) 2009-2012 Ingenescus. All Rights Reserved. 
//
//     This Work is being provided by the copyright holder(s) subject to the 
//     following license. By obtaining, using and/or copying this Work, you 
//     agree that you have read, understand, and will comply with the 
//     following terms and conditions. 
//     
//     Permission to use, copy and distribute unmodified versions of this 
//     Work, for any purpose, without fee or royalty is hereby granted, 
//     provided that you include the above copyright notices and the terms of 
//     this license on ALL copies of the Work or portions thereof. 
//     
//     You may modify or create Derivatives of this Work only for your 
//     internal purposes. You shall not distribute or transfer any such 
//     Derivative of this Work to any location or to any third party. For the 
//     purposes of this license, "Derivative" shall mean any derivative of the 
//     Work as defined in the United States Copyright Act of 1976, such as a 
//     translation or modification. 
//     
//     This Work and the information contained herein is provided on an "AS 
//     IS" basis WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING 
//     BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
//     PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS 
//     OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
//     LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
//     FROM, OUT OF OR IN CONNECTION WITH THE WORK OR THE USE OR OTHER 
//     DEALINGS IN THE WORK. 
//     
//     The export of software employing encryption technology may require a 
//     specific license from the United States Government. It is the 
//     responsibility of any person or organization contemplating export to 
//     obtain such a license before exporting this Work. 
//

package org.osid;


/**
 *  <p>The <code> OsidForm </code> is the vehicle used to create and update 
 *  objects. The form is a container for data to be sent to an update or 
 *  create method of a session. Applications should persist their own data 
 *  until a form is successfully submitted in an update or create transaction. 
 *  </p> 
 *  
 *  <p> The form may provide some feedback as to the validity of certain data 
 *  updates before the update transaction is issued to the correspodning 
 *  session but a successful modification of the form is not a guarantee of 
 *  success for the update transaction. A consumer may elect to perform all 
 *  updates within a single update transaction or break up a large update 
 *  intio smaller units. The tradeoff is the granularity of error feedback vs. 
 *  the performance gain of a single transaction. </p> 
 *  
 *  <p> <code> OsidForms </code> are <code> Identifiable. </code> The <code> 
 *  Id </code> of the <code> OsidForm </code> is used to uniquely identify the 
 *  update or create transaction and not that of the object being updated. 
 *  Currently, it is not necessary to have these <code> Ids </code> persisted. 
 *  </p> 
 *  
 *  <p> As with all aspects of the OSIDs, nulls cannot be used. Methods to 
 *  clear values are also defined in the form. </p> 
 *  
 *  <p> A new <code> OsidForm </code> should be acquired for each transaction 
 *  upon an <code> OsidObject. </code> Forms should not be reused from one 
 *  object to another even if the supplied data is the same as the forms may 
 *  encapsulate data specific to the object requested. Example of changing a 
 *  display name and a color defined in a color interface extension: </p> 
 *  
 *  <pre>
 *       
 *       
 *       ObjectForm form = session.getObjectFormForUpdate(objectId);
 *       form.setDisplayName("new name");
 *       ColorForm recordForm = form.getFormRecord(colorRecordType);
 *       recordForm.setColor("green");
 *       session.updateObject(objectId, form);
 *       
 *                   
 *       
 *  </pre>
 *  
 */

public interface OsidForm
    extends org.osid.Identifiable,
            org.osid.Suppliable {


    /**
     *  Tests if this form is for an update operation. 
     *
     *  @return <code> true </code> if this form is for an update operation, 
     *          <code> false </code> if for a create operation 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean isForUpdate();


    /**
     *  Gets a default locale for <code> DisplayTexts </code> when a locale is 
     *  not specified. 
     *
     *  @return the default locale 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.locale.Locale getDefaultLocale();


    /**
     *  Gets a list of locales for available <code> DisplayText </code> 
     *  translations that can be performed using this form. 
     *
     *  @return a list of available locales or an empty list if no translation 
     *          operations are available 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.locale.LocaleList getLocales();


    /**
     *  Specifies a language and script type for <code> DisplayText </code> 
     *  fields in this form. Setting a locale to something other than the 
     *  default locale may affect the <code> Metadata </code> in this form. 
     *  
     *  If multiple locales are available for managing translations, the 
     *  <code> Metadata </code> indicates the fields are unset as they may be 
     *  returning a defeult value based on the default locale. 
     *
     *  @param  languageType the language type 
     *  @param  scriptType the script type 
     *  @throws org.osid.NullArgumentException <code> languageType </code> or 
     *          <code> scriptType </code> is null 
     *  @throws org.osid.UnsupportedException <code> languageType </code> and 
     *          <code> scriptType </code> not available from <code> 
     *          getLocales() </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public void setLocale(org.osid.type.Type languageType, 
                          org.osid.type.Type scriptType);


    /**
     *  Gets the metadata for the comment corresponding to this form 
     *  submission. The comment is used for describing the nature of the 
     *  change to the corresponding object for the purposes of logging and 
     *  auditing. 
     *
     *  @return metadata for the comment 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.Metadata getJournalCommentMetadata();


    /**
     *  Sets a comment. 
     *
     *  @param  comment the new comment 
     *  @throws org.osid.InvalidArgumentException <code> comment </code> is 
     *          invalid 
     *  @throws org.osid.NoAccessException <code> Metadata.isReadonly() 
     *          </code> is <code> true </code> 
     *  @throws org.osid.NullArgumentException <code> comment </code> is 
     *          <code> null </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public void setJournalComment(String comment);


    /**
     *  Tests if ths form is in a valid state for submission. A form is valid 
     *  if all required data has been supplied compliant with any constraints. 
     *
     *  @return <code> false </code> if there is a known error in this form, 
     *          <code> true </code> otherwise 
     *  @throws org.osid.OperationFailedException attempt to perform 
     *          validation failed 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean isValid()
        throws org.osid.OperationFailedException;


    /**
     *  Gets text messages corresponding to additional instructions to pass 
     *  form validation. 
     *
     *  @return a list of messages 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.locale.DisplayText[] getValidationMessages();


    /**
     *  Gets a list of metadata for the elements in this form which are not 
     *  valid. 
     *
     *  @return invalid metadata 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.Metadata[] getInvalidMetadata();
}
