//
// org.osid.authentication.keys.KeyLookupSession
//
//     The OSID Java specification for org.osid.authentication.keys.KeyLookupSession.
//
// Copyright (C) 2009-2010 Ingenescus. All Rights Reserved. 
//
//     This Work is being provided by the copyright holder(s) subject to the 
//     following license. By obtaining, using and/or copying this Work, you 
//     agree that you have read, understand, and will comply with the 
//     following terms and conditions. 
//     
//     Permission to use, copy and distribute unmodified versions of this 
//     Work, for any purpose, without fee or royalty is hereby granted, 
//     provided that you include the above copyright notices and the terms of 
//     this license on ALL copies of the Work or portions thereof. 
//     
//     You may nodify or create Derivatives of this Work only for your 
//     internal purposes. You shall not distribute or transfer any such 
//     Derivative of this Work to any location or to any third party. For the 
//     purposes of this license, "Derivative" shall mean any derivative of the 
//     Work as defined in the United States Copyright Act of 1976, such as a 
//     translation or modification. 
//     
//     This Work and the information contained herein is provided on an "AS 
//     IS" basis WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING 
//     BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
//     PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS 
//     OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
//     LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
//     FROM, OUT OF OR IN CONNECTION WITH THE WORK OR THE USE OR OTHER 
//     DEALINGS IN THE WORK. 
//     
//     The export of software employing encryption technology may require a 
//     specific license from the United States Government. It is the 
//     responsibility of any person or organization contemplating export to 
//     obtain such a license before exporting this Work. 
//

package org.osid.authentication.keys;


/**
 *  <p>This session provides methods for retrieving <code> Key </code> 
 *  objects. The <code> Key </code> is associated with an <code> Agent </code> 
 *  and identified by the <code> Agent </code> Id. </p> 
 *  
 *  <p> This session defines two sets of views which offer differing behaviors 
 *  when retrieving multiple objects. </p> 
 *  
 *  <ul>
 *      <li> comparative view: elements may be silently omitted or re-ordered 
 *      </li> 
 *      <li> plenary view: provides a complete and ordered result set or is an 
 *      error condition </li> 
 *      <li> isolated agency view: All key methods in this session operate, 
 *      retrieve and pertain to keys defined explicitly in the current agency. 
 *      Using an isolated view is useful for managing keys with the 
 *      KeyAdminSession. </li> 
 *      <li> federated agency view: All key methods in this session operate, 
 *      retrieve and pertain to all keys defined in this agency and any other 
 *      agents implicitly available in this agency through agency inheritence. 
 *      </li> 
 *  </ul>
 *  
 *  
 *  <p> Generally, the comparative view should be used for most applications 
 *  as it permits operation even if there is data out of sync. For example, a 
 *  hierarchy output can be plugged into a lookup method to retrieve all 
 *  objects known to a hierarchy, but it may not be necessary to break 
 *  execution if a node from the hierarchy no longer exists. However, some 
 *  administrative applications may need to know whether it had retrieved an 
 *  entire set of objects and may sacrifice some interoperability for the sake 
 *  of precision. </p> 
 *  
 *  <p> Keys may have an additional records indicated by their respective 
 *  record types. The record may not be accessed through a cast of the <code> 
 *  Key. </code> </p>
 */

public interface KeyLookupSession
    extends org.osid.OsidSession {


    /**
     *  Gets the <code> Agency </code> <code> Id </code> associated with this 
     *  session. 
     *
     *  @return the <code> Agency Id </code> associated with this session 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.id.Id getAgencyId();


    /**
     *  Gets the <code> Agency </code> associated with this session. 
     *
     *  @return the <code> Agency </code> associated with this session 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.authentication.Agency getAgency()
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Tests if this user can lookup <code> Keys. </code> A return of true 
     *  does not guarantee successful authorization. A return of false 
     *  indicates that it is known getting a <code> Key </code> will result in 
     *  a <code> PERMISSION_DENIED. </code> This is intended as a hint to an 
     *  application that may opt to offer key management functions to 
     *  unauthorized users. 
     *
     *  @return <code> false </code> if key management is not authorized, 
     *          <code> true </code> otherwise 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean canLookupKeys();


    /**
     *  The returns from the lookup methods may omit or translate elements 
     *  based on this session, such as authorization, and not result in an 
     *  error. This view is used when greater interoperability is desired at 
     *  the expense of precision. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useComparativeKeyView();


    /**
     *  A complete view of the <code> Key </code> returns is desired. Methods 
     *  will return what is requested or result in an error. This view is used 
     *  when greater precision is desired at the expense of interoperability. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void usePlenaryKeyView();


    /**
     *  Federates the view for methods in this session. A federated view will 
     *  include keys in agencies which are children of this agency in the 
     *  agency hierarchy. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useFederatedAgencyView();


    /**
     *  Isolates the view for methods in this session. An isolated view 
     *  restricts lookups to this agency only. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useIsolatedAgencyView();


    /**
     *  Gets the agent key. In plenary mode, the exact Id is found or a 
     *  NOT_FOUND results. Otherwise, the returned <code> Agent </code> via 
     *  the <code> Key </code> may have a different Id than requested, such as 
     *  the case where a duplicate <code> Id </code> was assigned to an Agent 
     *  and retained for compatibility. 
     *
     *  @param  keyId the <code> Id </code> of the <code> Key </code> 
     *  @return the key 
     *  @throws org.osid.NotFoundException <code> keyId </code> is not found 
     *          or no key exists 
     *  @throws org.osid.NullArgumentException <code> keyId </code> is <code> 
     *          null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.authentication.keys.Key getKey(org.osid.id.Id keyId)
        throws org.osid.NotFoundException,
               org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a <code> KeyList </code> corresponding to the given agent <code> 
     *  IdList. </code> In plenary mode, the returned list contains all of the 
     *  keys for agents specified in the <code> Id </code> list, in the order 
     *  of the list, including duplicates, or an error results if an <code> Id 
     *  </code> in the supplied list is not found or inaccessible. Otherwise, 
     *  inaccessible <code> Keys </code> may be omitted from the list and may 
     *  present the elements in any order including returning a unique set. 
     *
     *  @param  keyIds the list of <code> Ids </code> to retrieve 
     *  @return the returned <code> Key </code> list 
     *  @throws org.osid.NotFoundException an <code> Id was </code> not found 
     *  @throws org.osid.NullArgumentException <code> keyIds </code> is <code> 
     *          null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.authentication.keys.KeyList getKeysByIds(org.osid.id.IdList keyIds)
        throws org.osid.NotFoundException,
               org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a <code> KeyList </code> corresponding to the given key genus 
     *  <code> Type </code> which does not include keys of genus types derived 
     *  from the specified <code> Type. </code> In plenary mode, the returned 
     *  list contains all known keys or an error results. Otherwise, the 
     *  returned list may contain only those keys that are accessible through 
     *  this session. 
     *
     *  @param  keyGenusType a key genus type 
     *  @return the returned <code> Key </code> list 
     *  @throws org.osid.NullArgumentException <code> keyGenusType </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *          occurred 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.authentication.keys.KeyList getKeysByGenusType(org.osid.type.Type keyGenusType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a <code> KeyList </code> corresponding to the given key genus 
     *  <code> Type </code> and include any additional keys with genus types 
     *  derived from the specified <code> Type. </code> In plenary mode, the 
     *  returned list contains all known keys or an error results. Otherwise, 
     *  the returned list may contain only those keys that are accessible 
     *  through this session. 
     *
     *  @param  keyGenusType a key genus type 
     *  @return the returned <code> Key </code> list 
     *  @throws org.osid.NullArgumentException <code> keyGenusType </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *          occurred 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.authentication.keys.KeyList getKeysByParentGenusType(org.osid.type.Type keyGenusType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of keys corresponding to the given key record <code> Type. 
     *  </code> The set of keys implementing the given record type is 
     *  returned. In plenary mode, the returned list contains all known keys 
     *  or an error results. Otherwise, the returned list may contain only 
     *  those keys that are accessible through this session. 
     *
     *  @param  keyRecordType a key record type 
     *  @return the returned <code> Key </code> list 
     *  @throws org.osid.NullArgumentException <code> keyRecordType </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.authentication.keys.KeyList getKeysByRecordType(org.osid.type.Type keyRecordType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets the agent key. 
     *
     *  @param  agentId the <code> Id </code> of the <code> Agent </code> 
     *  @return the key 
     *  @throws org.osid.NotFoundException <code> agentId </code> is not found 
     *          or no key exists 
     *  @throws org.osid.NullArgumentException <code> agentId </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.authentication.keys.Key getKeyForAgent(org.osid.id.Id agentId)
        throws org.osid.NotFoundException,
               org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets all <code> Keys. </code> In plenary mode, the returned list 
     *  contains all known keys or an error results. Otherwise, the returned 
     *  list may contain only those keys that are accessible through this 
     *  session. 
     *
     *  @return a list of <code> Keys </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.authentication.keys.KeyList getKeys()
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;
}
