//
// org.osid.authentication.process.TrustLookupSession
//
//     The OSID Java specification for org.osid.authentication.process.TrustLookupSession.
//
// Copyright (C) 2002-2004, 2007-2009 Massachusetts Institute of Technology. 
// All Rights Reserved. 
//
//     This Work is being provided by the copyright holder(s) subject to the 
//     following license. By obtaining, using and/or copying this Work, you 
//     agree that you have read, understand, and will comply with the 
//     following terms and conditions. 
//     
//     Permission to use, copy and distribute unmodified versions of this 
//     Work, for any purpose, without fee or royalty is hereby granted, 
//     provided that you include the above copyright notices and the terms of 
//     this license on ALL copies of the Work or portions thereof. 
//     
//     You may nodify or create Derivatives of this Work only for your 
//     internal purposes. You shall not distribute or transfer any such 
//     Derivative of this Work to any location or to any third party. For the 
//     purposes of this license, "Derivative" shall mean any derivative of the 
//     Work as defined in the United States Copyright Act of 1976, such as a 
//     translation or modification. 
//     
//     This Work and the information contained herein is provided on an "AS 
//     IS" basis WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING 
//     BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
//     PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS 
//     OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
//     LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
//     FROM, OUT OF OR IN CONNECTION WITH THE WORK OR THE USE OR OTHER 
//     DEALINGS IN THE WORK. 
//     
//     The export of software employing encryption technology may require a 
//     specific license from the United States Government. It is the 
//     responsibility of any person or organization contemplating export to 
//     obtain such a license before exporting this Work. 
//

package org.osid.authentication.process;


/**
 *  <p>This session provides methods for retrieving <code> Trusts. </code> The 
 *  <code> Trust </code> represents the trust level of an agent. The 
 *  relationship among <code> Agents </code> and <code> Trust </code> is not 
 *  explicity managed but understood by an Authentication OSID Provider when 
 *  orchestration to an Authorization OSID Provider is desired. </p> 
 *  
 *  <p> This session defines two sets of views which offer differing behaviors 
 *  when retrieving multiple objects. </p> 
 *  
 *  <ul>
 *      <li> comparative view: elements may be silently omitted or re-ordered 
 *      </li> 
 *      <li> plenary view: provides a complete and ordered result set or is an 
 *      error condition </li> 
 *      <li> isolated agency view: All agent methods in this session operate, 
 *      retrieve and pertain to trusts defined explicitly in the current 
 *      agency. </li> 
 *      <li> federated agency view: All trust methods in this session operate, 
 *      retrieve and pertain to all trusts defined in this agency and any 
 *      other trusts implicitly available in this agency through agency 
 *      inheritence. </li> 
 *  </ul>
 *  
 *  
 *  <p> Generally, the comparative view should be used for most applications 
 *  as it permits operation even if there a particular element is 
 *  inaccessible. For example, a hierarchy output can be plugged into a lookup 
 *  method to retrieve all objects known to a hierarchy, but it may not be 
 *  necessary to break execution if a node from the hierarchy no longer 
 *  exists. However, some administrative applications may need to know whether 
 *  it had retrieved an entire set of objects and may sacrifice some 
 *  interoperability for the sake of precision. </p> 
 *  
 *  <p> Trusts may have an additional records indicated by their respective 
 *  record types. The record may not be accessed through a cast of the <code> 
 *  Trust. </code> </p>
 */

public interface TrustLookupSession
    extends org.osid.OsidSession {


    /**
     *  Gets the <code> Agency </code> <code> Id </code> associated with this 
     *  session. 
     *
     *  @return the <code> Agency Id </code> associated with this session 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.id.Id getAgencyId();


    /**
     *  Gets the <code> Agency </code> associated with this session. 
     *
     *  @return the <code> Agency </code> associated with this session 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.authentication.Agency getAgency()
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Tests if this user can perform <code> Trust </code> lookups. A return 
     *  of true does not guarantee successful authorization. A return of false 
     *  indicates that it is known all methods in this session will result in 
     *  a <code> PERMISSION_DENIED. </code> This is intended as a hint to an 
     *  application that may opt not to offer lookup operations. 
     *
     *  @return <code> false </code> if lookup methods are not authorized, 
     *          <code> true </code> otherwise 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean canLookupTrusts();


    /**
     *  The returns from the lookup methods may omit or translate elements 
     *  based on this session, such as authorization, and not result in an 
     *  error. This view is used when greater interoperability is desired at 
     *  the expense of precision. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useComparativeTrustView();


    /**
     *  A complete view of the <code> Trust </code> returns is desired. 
     *  Methods will return what is requested or result in an error. This view 
     *  is used when greater precision is desired at the expense of 
     *  interoperability. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void usePlenaryTrustView();


    /**
     *  Federates the view for methods in this session. A federated view will 
     *  include trusts in agencies which are children of this agency in the 
     *  agency hierarchy. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useFederatedAgencyView();


    /**
     *  Isolates the view for methods in this session. An isolated view 
     *  restricts lookups to this agency only. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useIsolatedAgencyView();


    /**
     *  Gets the <code> Trust </code> specified by its <code> Id. </code> In 
     *  plenary mode, the exact <code> Id </code> is found or a <code> 
     *  NOT_FOUND </code> results. Otherwise, the returned <code> Trust 
     *  </code> may have a different <code> Id </code> than requested, such as 
     *  the case where a duplicate <code> Id </code> was assigned to a <code> 
     *  Trust </code> and retained for compatibility. 
     *
     *  @param  trustId the <code> Id </code> of the <code> Trust </code> to 
     *          retrieve 
     *  @return the returned <code> Trust </code> 
     *  @throws org.osid.NotFoundException no <code> Trust </code> found with 
     *          the given <code> Id </code> 
     *  @throws org.osid.NullArgumentException <code> trustId </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.authentication.process.Trust getTrust(org.osid.id.Id trustId)
        throws org.osid.NotFoundException,
               org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a <code> TrustList </code> corresponding to the given <code> 
     *  IdList. </code> In plenary mode, the returned list contains all of the 
     *  trusts specified in the <code> Id </code> list, in the order of the 
     *  list, including duplicates, or an error results if an <code> Id 
     *  </code> in the supplied list is not found or inaccessible. Otherwise, 
     *  inaccessible <code> Trusts </code> may be omitted from the list and 
     *  may present the elements in any order including returning a unique 
     *  set. 
     *
     *  @param  trustIds a list of trust <code> Ids </code> 
     *  @return the returned <code> Trust list </code> 
     *  @throws org.osid.NotFoundException an <code> Id was </code> not found 
     *  @throws org.osid.NullArgumentException <code> trustIds </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.authentication.process.TrustList getTrustsByIds(org.osid.id.IdList trustIds)
        throws org.osid.NotFoundException,
               org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a <code> TrustList </code> corresponding to the given trust genus 
     *  <code> Type </code> which does not include trusts of genus types 
     *  derived from the specified <code> Type. </code> In plenary mode, the 
     *  returned list contains all known trusts or an error results. 
     *  Otherwise, the returned list may contain only those trusts that are 
     *  accessible through this session. 
     *
     *  @param  trustGenusType a trust genus type 
     *  @return the returned <code> Trust </code> list 
     *  @throws org.osid.NullArgumentException <code> trustGenusType </code> 
     *          is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.authentication.process.TrustList getTrustsByGenusType(org.osid.type.Type trustGenusType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a <code> TrustList </code> corresponding to the given trust genus 
     *  <code> Type </code> and include any additional trusts with genus types 
     *  derived from the specified <code> Type. </code> In plenary mode, the 
     *  returned list contains all known trusts or an error results. 
     *  Otherwise, the returned list may contain only those trusts that are 
     *  accessible through this session. 
     *
     *  @param  trustGenusType a trust genus type 
     *  @return the returned <code> Trust </code> list 
     *  @throws org.osid.NullArgumentException <code> trustGenusType </code> 
     *          is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.authentication.process.TrustList getTrustsByParentGenusType(org.osid.type.Type trustGenusType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a <code> TrustList </code> containing the given trust record 
     *  <code> Type. </code> In plenary mode, the returned list contains all 
     *  known trusts or an error results. Otherwise, the returned list may 
     *  contain only those trusts that are accessible through this session. 
     *
     *  @param  trustRecordType a trust record type 
     *  @return the returned <code> Trust </code> list 
     *  @throws org.osid.NullArgumentException <code> trustRecordType </code> 
     *          is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.authentication.process.TrustList getTrustsByRecordType(org.osid.type.Type trustRecordType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a <code> TrustList </code> in the same circle, or the same level 
     *  of confidence, as the given trust. In plenary mode, the returned list 
     *  contains all known trusts or an error results. Otherwise, the returned 
     *  list may contain only those trusts that are accessible through this 
     *  session. 
     *
     *  @param  trustId a trust <code> Id </code> 
     *  @return the returned <code> Trust </code> list 
     *  @throws org.osid.NotFoundException <code> trustId </code> is not found 
     *  @throws org.osid.NullArgumentException <code> trustId </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.authentication.process.TrustList getCircleOfTrust(org.osid.id.Id trustId)
        throws org.osid.NotFoundException,
               org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets all <code> Trusts. </code> In plenary mode, the returned list 
     *  contains all known trusts or an error results. Otherwise, the returned 
     *  list may contain only those trusts that are accessible through this 
     *  session. 
     *
     *  @return a list of <code> Trusts </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.authentication.process.TrustList getTrusts()
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;
}
