//
// org.osid.billing.CustomerLookupSession
//
//     The OSID Java specification for org.osid.billing.CustomerLookupSession.
//
// Copyright (C) 2012 Ingenescus. All Rights Reserved. 
//
//     This Work is being provided by the copyright holder(s) subject to the 
//     following license. By obtaining, using and/or copying this Work, you 
//     agree that you have read, understand, and will comply with the 
//     following terms and conditions. 
//     
//     Permission to use, copy and distribute unmodified versions of this 
//     Work, for any purpose, without fee or royalty is hereby granted, 
//     provided that you include the above copyright notices and the terms of 
//     this license on ALL copies of the Work or portions thereof. 
//     
//     You may modify or create Derivatives of this Work only for your 
//     internal purposes. You shall not distribute or transfer any such 
//     Derivative of this Work to any location or to any third party. For the 
//     purposes of this license, "Derivative" shall mean any derivative of the 
//     Work as defined in the United States Copyright Act of 1976, such as a 
//     translation or modification. 
//     
//     This Work and the information contained herein is provided on an "AS 
//     IS" basis WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING 
//     BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
//     PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS 
//     OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
//     LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
//     FROM, OUT OF OR IN CONNECTION WITH THE WORK OR THE USE OR OTHER 
//     DEALINGS IN THE WORK. 
//     
//     The export of software employing encryption technology may require a 
//     specific license from the United States Government. It is the 
//     responsibility of any person or organization contemplating export to 
//     obtain such a license before exporting this Work. 
//

package org.osid.billing;


/**
 *  <p>This session defines methods for retrieving customers. A <code> 
 *  Customer </code> is a <code> Resource </code> related to the billing 
 *  system. </p> 
 *  
 *  <p> This lookup session defines several views: </p> 
 *  
 *  <ul>
 *      <li> comparative view: elements may be silently omitted or re-ordered 
 *      </li> 
 *      <li> plenary view: provides a complete result set or is an error 
 *      condition </li> 
 *      <li> isolated business view: All customer methods in this session 
 *      operate, retrieve and pertain to customers defined explicitly in the 
 *      current business. Using an isolated view is useful for managing <code> 
 *      Customers </code> with the <code> CustomerAdminSession. </code> </li> 
 *      <li> federated business view: All customer lookup methods in this 
 *      session operate, retrieve and pertain to all customers defined in this 
 *      business and any other customers implicitly available in this business 
 *      through business inheritence. </li> 
 *      <li> effective customer view: All customer lookup methods return 
 *      customers where the current date falls in between the effective dates 
 *      inclusive. </li> 
 *      <li> any effective customer view: Customers of any effective date are 
 *      returned from methods. </li> 
 *  </ul>
 *  
 *  
 *  <p> The methods <code> useFederatedBusinessView() </code> and <code> 
 *  useIsolatedBusinessView() </code> behave as a radio group and one should 
 *  be selected before invoking any lookup methods. </p> 
 *  
 *  <p> Customers may have an additional records indicated by their respective 
 *  record types. The record may not be accessed through a cast of the <code> 
 *  Customer. </code> </p>
 */

public interface CustomerLookupSession
    extends org.osid.OsidSession {


    /**
     *  Gets the <code> Business </code> <code> Id </code> associated with 
     *  this session. 
     *
     *  @return the <code> Business Id </code> associated with this session 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.id.Id getBusinessId();


    /**
     *  Gets the <code> Business </code> associated with this session. 
     *
     *  @return the business 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.billing.Business getBusiness()
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Tests if this user can perform <code> Customer </code> lookups. A 
     *  return of true does not guarantee successful authorization. A return 
     *  of false indicates that it is known all methods in this session will 
     *  result in a <code> PERMISSION_DENIED. </code> This is intended as a 
     *  hint to an application that may not offer lookup operations to 
     *  unauthorized users. 
     *
     *  @return <code> false </code> if lookup methods are not authorized, 
     *          <code> true </code> otherwise 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean canLookupCustomers();


    /**
     *  The returns from the lookup methods may omit or translate elements 
     *  based on this session, such as authorization, and not result in an 
     *  error. This view is used when greater interoperability is desired at 
     *  the expense of precision. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useComparativeCustomerView();


    /**
     *  A complete view of the <code> Customer </code> returns is desired. 
     *  Methods will return what is requested or result in an error. This view 
     *  is used when greater precision is desired at the expense of 
     *  interoperability. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void usePlenaryCustomerView();


    /**
     *  Federates the view for methods in this session. A federated view will 
     *  include customers in catalogs which are children of this catalog in 
     *  the business hierarchy. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useFederatedBusinessView();


    /**
     *  Isolates the view for methods in this session. An isolated view 
     *  restricts retrievals to this business only. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useIsolatedBusinessView();


    /**
     *  Only customers whose effective dates are current are returned by 
     *  methods in this session. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useEffectiveCustomerView();


    /**
     *  All customers of any effective dates are returned by methods in this 
     *  session. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useAnyEffectiveCustomerView();


    /**
     *  Gets the <code> Customer </code> specified by its <code> Id. </code> 
     *  
     *  <code> </code> In plenary mode, the exact <code> Id </code> is found 
     *  or a <code> NOT_FOUND </code> results. Otherwise, the returned <code> 
     *  Customer </code> may have a different <code> Id </code> than 
     *  requested, such as the case where a duplicate <code> Id </code> was 
     *  assigned to a <code> Customer </code> and retained for compatibility. 
     *  
     *  In effective mode, customers are returned that are currently 
     *  effective. In any effective mode, effective customers and those 
     *  currently expired are returned. 
     *
     *  @param  customerId the <code> Id </code> of the <code> Customer 
     *          </code> to retrieve 
     *  @return the returned <code> Customer </code> 
     *  @throws org.osid.NotFoundException no <code> Customer </code> found 
     *          with the given <code> Id </code> 
     *  @throws org.osid.NullArgumentException <code> customerId </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.billing.Customer getCustomer(org.osid.id.Id customerId)
        throws org.osid.NotFoundException,
               org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a <code> CustomerList </code> corresponding to the given <code> 
     *  IdList. </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all of the 
     *  customers specified in the <code> Id </code> list, in the order of the 
     *  list, including duplicates, or an error results if an <code> Id 
     *  </code> in the supplied list is not found or inaccessible. Otherwise, 
     *  inaccessible <code> Customers </code> may be omitted from the list and 
     *  may present the elements in any order including returning a unique 
     *  set. 
     *  
     *  In effective mode, customers are returned that are currently 
     *  effective. In any effective mode, effective customers and those 
     *  currently expired are returned. 
     *
     *  @param  customerIds the list of <code> Ids </code> to retrieve 
     *  @return the returned <code> CustomerList </code> list 
     *  @throws org.osid.NotFoundException an <code> Id was </code> not found 
     *  @throws org.osid.NullArgumentException <code> customerIds </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.billing.CustomerList getCustomersByIds(org.osid.id.IdList customerIds)
        throws org.osid.NotFoundException,
               org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a <code> CustomerList </code> corresponding to the given customer 
     *  genus <code> Type </code> which does not include customers of types 
     *  derived from the specified <code> Type. </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  customers or an error results. Otherwise, the returned list may 
     *  contain only those customers that are accessible through this session. 
     *  
     *  In effective mode, customers are returned that are currently 
     *  effective. In any effective mode, effective customers and those 
     *  currently expired are returned. 
     *
     *  @param  customerGenusType a customer genus type 
     *  @return the returned <code> CustomerList </code> list 
     *  @throws org.osid.NullArgumentException <code> customerGenusType 
     *          </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.billing.CustomerList getCustomersByGenusType(org.osid.type.Type customerGenusType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a <code> CustomerList </code> corresponding to the given customer 
     *  genus <code> Type </code> and include any additional customers with 
     *  genus types derived from the specified <code> Type. </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  customers or an error results. Otherwise, the returned list may 
     *  contain only those customers that are accessible through this session. 
     *  
     *  In effective mode, customers are returned that are currently 
     *  effective. In any effective mode, effective customers and those 
     *  currently expired are returned. 
     *
     *  @param  customerGenusType a customer genus type 
     *  @return the returned <code> CustomerList </code> list 
     *  @throws org.osid.NullArgumentException <code> customerGenusType 
     *          </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.billing.CustomerList getCustomersByParentGenusType(org.osid.type.Type customerGenusType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a <code> CustomerList </code> containing the given customer 
     *  record <code> Type. </code> 
     *  
     *  In plenary mode, the returned list contains all known customers or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  customers that are accessible through this session. 
     *  
     *  In effective mode, customers are returned that are currently 
     *  effective. In any effective mode, effective customers and those 
     *  currently expired are returned. 
     *
     *  @param  customerRecordType a customer record type 
     *  @return the returned <code> CustomerList </code> list 
     *  @throws org.osid.NullArgumentException <code> customerRecordType 
     *          </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.billing.CustomerList getCustomersByRecordType(org.osid.type.Type customerRecordType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a <code> CustomerList </code> effective during the entire given 
     *  date range inclusive but not confined to the date range. 
     *  
     *  In plenary mode, the returned list contains all known customers or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  customers that are accessible through this session. 
     *  
     *  In effective mode, customers are returned that are currently effective 
     *  in addition to being effective during the given date range. In any 
     *  effective mode, effective customers and those currently expired are 
     *  returned. 
     *
     *  @param  from starting date 
     *  @param  to ending date 
     *  @return the returned <code> CustomerList </code> list 
     *  @throws org.osid.InvalidArgumentException <code> from </code> is 
     *          greater than <code> to </code> 
     *  @throws org.osid.NullArgumentException <code> from </code> or <code> 
     *          to </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.billing.CustomerList getCustomersOnDate(org.osid.calendaring.DateTime from, 
                                                            org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a <code> CustomerList </code> related to the given customer 
     *  number. 
     *  
     *  In plenary mode, the returned list contains all known customers or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  customers that are accessible through this session. 
     *  
     *  In effective mode, customers are returned that are currently 
     *  effective. In any effective mode, effective customers and those 
     *  currently expired are returned. 
     *
     *  @param  number a customer number 
     *  @return the returned <code> CustomerList </code> list 
     *  @throws org.osid.NullArgumentException <code> number </code> is <code> 
     *          null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.billing.CustomerList getCustomersByNumber(String number)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a <code> CustomerList </code> related to the given resource 
     *  <code> . </code> 
     *  
     *  In plenary mode, the returned list contains all known customers or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  customers that are accessible through this session. 
     *  
     *  In effective mode, customers are returned that are currently 
     *  effective. In any effective mode, effective customers and those 
     *  currently expired are returned. 
     *
     *  @param  resourceId a resource <code> Id </code> 
     *  @return the returned <code> CustomerList </code> list 
     *  @throws org.osid.NullArgumentException <code> resourceId </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.billing.CustomerList getCustomersByResource(org.osid.id.Id resourceId)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a <code> CustomerList </code> of the given resource and effective 
     *  during the entire given date range inclusive but not confined to the 
     *  date range. 
     *  
     *  In plenary mode, the returned list contains all known customers or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  customers that are accessible through this session. 
     *  
     *  In effective mode, customers are returned that are currently effective 
     *  in addition to being effective during the given date range. In any 
     *  effective mode, effective customers and those currently expired are 
     *  returned. 
     *
     *  @param  resourceId a resource <code> Id </code> 
     *  @param  from starting date 
     *  @param  to ending date 
     *  @return the returned <code> CustomerList </code> list 
     *  @throws org.osid.InvalidArgumentException <code> from </code> is 
     *          greater than <code> to </code> 
     *  @throws org.osid.NullArgumentException <code> resourceId, from, 
     *          </code> or <code> to </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.billing.CustomerList getCustomersByResourceOnDate(org.osid.id.Id resourceId, 
                                                                      org.osid.calendaring.DateTime from, 
                                                                      org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a <code> CustomerList </code> having the given activity. 
     *  
     *  In plenary mode, the returned list contains all known customers or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  customers that are accessible through this session. 
     *  
     *  In effective mode, customers are returned that are currently 
     *  effective. In any effective mode, effective customers and those 
     *  currently expired are returned. 
     *
     *  @param  activityId an activity <code> Id </code> 
     *  @return the returned <code> CustomerList </code> list 
     *  @throws org.osid.NullArgumentException <code> activityId </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.billing.CustomerList getCustomersByActivity(org.osid.id.Id activityId)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets all <code> Customers. </code> In plenary mode, the returned list 
     *  contains all known customers or an error results. Otherwise, the 
     *  returned list may contain only those customers that are accessible 
     *  through this session. 
     *
     *  @return a list of <code> Customers </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.billing.CustomerList getCustomers()
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;
}
