//
// org.osid.calendaring.OffsetEvent
//
//     The OSID Java specification for org.osid.calendaring.OffsetEvent.
//
// Copyright (C) 2002-2004 Massachusetts Institute of Technology. 
// 
// Copyright (C) 2008-2011 Ingenescus. All Rights Reserved. 
//
//     This Work is being provided by the copyright holder(s) subject to the 
//     following license. By obtaining, using and/or copying this Work, you 
//     agree that you have read, understand, and will comply with the 
//     following terms and conditions. 
//     
//     Permission to use, copy and distribute unmodified versions of this 
//     Work, for any purpose, without fee or royalty is hereby granted, 
//     provided that you include the above copyright notices and the terms of 
//     this license on ALL copies of the Work or portions thereof. 
//     
//     You may modify or create Derivatives of this Work only for your 
//     internal purposes. You shall not distribute or transfer any such 
//     Derivative of this Work to any location or to any third party. For the 
//     purposes of this license, "Derivative" shall mean any derivative of the 
//     Work as defined in the United States Copyright Act of 1976, such as a 
//     translation or modification. 
//     
//     This Work and the information contained herein is provided on an "AS 
//     IS" basis WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING 
//     BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
//     PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS 
//     OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
//     LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
//     FROM, OUT OF OR IN CONNECTION WITH THE WORK OR THE USE OR OTHER 
//     DEALINGS IN THE WORK. 
//     
//     The export of software employing encryption technology may require a 
//     specific license from the United States Government. It is the 
//     responsibility of any person or organization contemplating export to 
//     obtain such a license before exporting this Work. 
//

package org.osid.calendaring;


/**
 *  <p>An <code> OffsetEvent </code> describes events that are relative to 
 *  other events. <code> OffsetEvents </code> may have a fixed or rubber start 
 *  date as well as a fixed or rubber end date. A rubber start and end date 
 *  has a variable duration based on the starting and ending milestones. An 
 *  <code> OffsetEvent </code> with a fixed start date and a fixed duration 
 *  behaves like a normal fixed <code> Event. </code> </p> 
 *  
 *  <p> Creating an <code> OffsetEvent </code> produces an <code> Event 
 *  </code> that is based on these offset rules. </p>
 */

public interface OffsetEvent
    extends org.osid.OsidRule {


    /**
     *  Tests if this offset event is based on a fixed start time. 
     *
     *  @return <code> true </code> if this offset is based on a fixed start 
     *          time, <code> false </code> otherwise 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean hasFixedStartTime();


    /**
     *  Gets the fixed start time for this event. 
     *
     *  @return the fixed start time 
     *  @throws org.osid.IllegalStateException <code> hasFixedStartIme() 
     *          </code> is <code> false </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.calendaring.DateTime getFixedStartTime();


    /**
     *  Gets the <code> Event Id </code> to which the start of this event is 
     *  offset. 
     *
     *  @return the relative event <code> Id </code> 
     *  @throws org.osid.IllegalStateException <code> hasFixedStartTime() 
     *          </code> is <code> true </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.id.Id getStartReferenceEventId();


    /**
     *  Gets the <code> Event </code> to which the start of this event is 
     *  offset. 
     *
     *  @return the relative event 
     *  @throws org.osid.IllegalStateException <code> hasFixedStartTime() 
     *          </code> is <code> true </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.calendaring.Event getStartReferenceEvent()
        throws org.osid.OperationFailedException;


    /**
     *  Tests if this offset start time based on a fixed period of time. 
     *
     *  @return <code> true </code> if this offset is based on a fixed period 
     *          of time, <code> false </code> otherwise 
     *  @throws org.osid.IllegalStateException <code> hasFixedStartTime() 
     *          </code> is <code> true </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean hasFixedStartOffset();


    /**
     *  Gets the fixed starting time offset. 
     *
     *  @return the offset 
     *  @throws org.osid.IllegalStateException <code> hasFixedStartOffset() 
     *          </code> is <code> false </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.calendaring.Duration getFixedStartOffset();


    /**
     *  Tests if this starting time offset is a based on a relative weekday. 
     *
     *  @return <code> true </code> if this offset is based on a relative 
     *          weekday, <code> false </code> otherwise 
     *  @throws org.osid.IllegalStateException <code> hasFixedStartOffset() 
     *          </code> is <code> true </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean hasRelativeWeekdayStartOffset();


    /**
     *  Gets the starting offset as the nth weekday from the relative event. 
     *  Zero is no offset. 
     *
     *  @return the offset 
     *  @throws org.osid.IllegalStateException <code> 
     *          hasRelativeWeekdayStartOffset() </code> is <code> false 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public long getRelativeWeekdayStartOffset();


    /**
     *  Gets the starting weekday number. The weekday is based on the calendar 
     *  type. On the Gregorian calendar, 0 is Sunday. 
     *
     *  @return the weekday number 
     *  @throws org.osid.IllegalStateException <code> 
     *          hasRelativeWeekdayStartOffset() </code> is <code> false 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public long getRelativeStartWeekday();


    /**
     *  Tests if this offset event is based on a fixed duration. 
     *
     *  @return <code> true </code> if this offset is based on a fixed 
     *          duration, <code> false </code> otherwise 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean hasFixedDuration();


    /**
     *  Gets the duration of the offset event. 
     *
     *  @param  units the units of the duration 
     *  @return the duration 
     *  @throws org.osid.IllegalStateException <code> hasFixedDuration() 
     *          </code> is <code> false </code> 
     *  @throws org.osid.NullArgumentException <code> units </code> is <code> 
     *          null </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.calendaring.Duration getFixedDuration(org.osid.calendaring.DateTimeResolution units);


    /**
     *  Gets the <code> Event Id </code> to which the end of this event is 
     *  offset. 
     *
     *  @return the relative event <code> Id </code> 
     *  @throws org.osid.IllegalStateException <code> hasFixedDuration() 
     *          </code> is <code> true </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.id.Id getEndReferenceEventId();


    /**
     *  Gets the <code> Event </code> to which the end of this event is 
     *  offset. 
     *
     *  @return the relative event 
     *  @throws org.osid.IllegalStateException <code> hasFixedDuration() 
     *          </code> is <code> true </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.calendaring.Event getEndReferenceEvent()
        throws org.osid.OperationFailedException;


    /**
     *  Tests if this offset end time based on a fixed period of time. 
     *
     *  @return <code> true </code> if this offset is based on a fixed period 
     *          of time, <code> false </code> otherwise 
     *  @throws org.osid.IllegalStateException <code> hasFixedDuration() 
     *          </code> is <code> true </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean hasFixedEndOffset();


    /**
     *  Gets the fixed ending time offset. 
     *
     *  @return the offset 
     *  @throws org.osid.IllegalStateException <code> hasFixedEndOffset() 
     *          </code> is <code> false </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.calendaring.Duration getFixedEndOffset();


    /**
     *  Tests if this ending time offset is a based on a relative weekday. 
     *
     *  @return <code> true </code> if this offset is based on a relative 
     *          weekday, <code> false </code> otherwise 
     *  @throws org.osid.IllegalStateException <code> hasFixedEndOffset() 
     *          </code> is <code> true </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean hasRelativeWeekdayEndOffset();


    /**
     *  Gets the ending offset as the nth weekday from the relative event. 
     *  Zero is no offset. 
     *
     *  @return the offset 
     *  @throws org.osid.IllegalStateException <code> 
     *          hasRelativeWeekdayEndOffset() </code> is <code> false </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public long getRelativeWeekdayEndOffset();


    /**
     *  Gets the ending weekday number. The weekday is based on the calendar 
     *  type. On the Gregorian calendar, 0 is Sunday. 
     *
     *  @return the weekday number 
     *  @throws org.osid.IllegalStateException <code> 
     *          hasRelativeWeekdayEndOffset() </code> is <code> false </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public long getRelativeEndWeekday();


    /**
     *  Gets a descriptive location. 
     *
     *  @return the location 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.locale.DisplayText getLocationDescription();


    /**
     *  Tests if a location is associated with this event. 
     *
     *  @return <code> true </code> if there is an associated location, <code> 
     *          false </code> otherwise 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean hasLocation();


    /**
     *  Gets the location <code> Id </code> . 
     *
     *  @return a location <code> Id </code> 
     *  @throws org.osid.IllegalStateException <code> hasLocation() </code> is 
     *          <code> false </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.id.Id getLocationId();


    /**
     *  Gets the <code> Location. </code> 
     *
     *  @return a location 
     *  @throws org.osid.IllegalStateException <code> hasLocation() </code> is 
     *          <code> false </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.mapping.Location getLocation()
        throws org.osid.OperationFailedException;


    /**
     *  Tests if a sponsor is associated with this event. 
     *
     *  @return <code> true </code> if there is an associated sponsor. <code> 
     *          false </code> otherwise 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean hasSponsors();


    /**
     *  Gets the <code> Id </code> of the event sponsors. 
     *
     *  @return the sponsor <code> Ids </code> 
     *  @throws org.osid.IllegalStateException <code> hasSponsors() </code> is 
     *          <code> false </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.id.IdList getSponsorIds();


    /**
     *  Gets the <code> Sponsors. </code> 
     *
     *  @return the sponsors 
     *  @throws org.osid.IllegalStateException <code> hasSponsors() </code> is 
     *          <code> false </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.resource.ResourceList getSponsors()
        throws org.osid.OperationFailedException;


    /**
     *  Gets the offset event record corresponding to the given <code> 
     *  OffsetEvent </code> record <code> Type. </code> This method is used to 
     *  retrieve an object implementing the requested record. The <code> 
     *  offsetEventRecordType </code> may be the <code> Type </code> returned 
     *  in <code> getRecordTypes() </code> or any of its parents in a <code> 
     *  Type </code> hierarchy where <code> 
     *  hasRecordType(offsetEventRecordType) </code> is <code> true </code> . 
     *
     *  @param  offsetEventRecordType the type of the record to retrieve 
     *  @return the offset event record 
     *  @throws org.osid.NullArgumentException <code> offsetEventRecordType 
     *          </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.UnsupportedException <code> 
     *          hasRecordType(offsetEventRecordType) </code> is <code> false 
     *          </code> 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.calendaring.records.OffsetEventRecord getOffsetEventRecord(org.osid.type.Type offsetEventRecordType)
        throws org.osid.OperationFailedException;
}
