//
// org.osid.configuration.ParameterLookupSession
//
//     The OSID Java specification for org.osid.configuration.ParameterLookupSession.
//
// Copyright (C) 2009-2010 Ingenescus. All Rights Reserved. 
//
//     This Work is being provided by the copyright holder(s) subject to the 
//     following license. By obtaining, using and/or copying this Work, you 
//     agree that you have read, understand, and will comply with the 
//     following terms and conditions. 
//     
//     Permission to use, copy and distribute unmodified versions of this 
//     Work, for any purpose, without fee or royalty is hereby granted, 
//     provided that you include the above copyright notices and the terms of 
//     this license on ALL copies of the Work or portions thereof. 
//     
//     You may nodify or create Derivatives of this Work only for your 
//     internal purposes. You shall not distribute or transfer any such 
//     Derivative of this Work to any location or to any third party. For the 
//     purposes of this license, "Derivative" shall mean any derivative of the 
//     Work as defined in the United States Copyright Act of 1976, such as a 
//     translation or modification. 
//     
//     This Work and the information contained herein is provided on an "AS 
//     IS" basis WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING 
//     BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
//     PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS 
//     OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
//     LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
//     FROM, OUT OF OR IN CONNECTION WITH THE WORK OR THE USE OR OTHER 
//     DEALINGS IN THE WORK. 
//     
//     The export of software employing encryption technology may require a 
//     specific license from the United States Government. It is the 
//     responsibility of any person or organization contemplating export to 
//     obtain such a license before exporting this Work. 
//

package org.osid.configuration;


/**
 *  <p>This session is used to retrieve parameters from a configuration 
 *  registry of parameters. </p> 
 *  
 *  <p> This lookup session defines several views. </p> 
 *  
 *  <ul>
 *      <li> comparative view: elements may be silently omitted or re-ordered 
 *      </li> 
 *      <li> plenary view: provides a complete and ordered result set or is an 
 *      error condition </li> 
 *      <li> isolated configuration view: All parameter methods in this 
 *      session operate, retrieve and pertain to parameters defined explicitly 
 *      in the current configuration. Using an isolated view is useful for 
 *      managing <code> Parameters </code> with the <code> 
 *      ParameterAdminSession. </code> </li> 
 *      <li> federated configuration view; All parameter methods in this 
 *      session operate, retrieve and pertain to all parameters defined in 
 *      this configuration and any other parameters implicitly available in 
 *      this configuration through configuration inheritence. </li> 
 *      <li> active parameter view: All parameter lookup methods return active 
 *      parameters. </li> 
 *      <li> any status parameter view: Parameters of any active or inactive 
 *      status are returned from methods. </li> 
 *  </ul>
 *  
 *  
 *  <p> The methods <code> federateParameterView() </code> and <code> 
 *  isolateParameterView() </code> behave as a radio group and one should be 
 *  selected before invoking any lookup methods. </p>
 */

public interface ParameterLookupSession
    extends org.osid.OsidSession {


    /**
     *  Gets the <code> Configuration </code> <code> Id </code> associated 
     *  with this session. 
     *
     *  @return the <code> Configuration Id </code> associated with this 
     *          session 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.id.Id getConfigurationId();


    /**
     *  Gets the <code> Configuration </code> associated with this session. 
     *
     *  @return the <code> Configuration </code> associated with this session 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.configuration.Configuration getConfiguration()
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Tests if this user can perform <code> Parameter </code> lookups. A 
     *  return of true does not guarantee successful authorization. A return 
     *  of false indicates that it is known all methods in this session will 
     *  result in a <code> PERMISSION_DENIED. </code> This is intended as a 
     *  hint to an application that may opt not to offer lookup operations to 
     *  unauthorized users. 
     *
     *  @return <code> false </code> if lookup methods are not authorized, 
     *          <code> true </code> otherwise 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean canLookupParameters();


    /**
     *  The returns from the lookup methods may omit or translate elements 
     *  based on this session, such as authorization, and not result in an 
     *  error. This view is used when greater interoperability is desired at 
     *  the expense of precision. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useComparativeParameterView();


    /**
     *  A complete view of the <code> Parameter </code> returns is desired. 
     *  Methods will return what is requested or result in an error. This view 
     *  is used when greater precision is desired at the expense of 
     *  interoperability. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void usePlenaryParameterView();


    /**
     *  Federates the view for methods in this session. A federated view will 
     *  include paramaters from parent configurations in the configuration 
     *  hierarchy. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useFederatedConfigurationView();


    /**
     *  Isolates the view for methods in this session. An isolated view 
     *  restricts lookups to this configuration only. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useIsolatedConfigurationView();


    /**
     *  Only active parameters are returned by methods in this session. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useActiveParameterView();


    /**
     *  All active and inactive parameters are returned by methods in this 
     *  session. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useAnyStatusParameterView();


    /**
     *  Gets the <code> Parameter </code> specified by its <code> Id. </code> 
     *  
     *  <code> </code> In plenary mode, the exact <code> Id </code> is found 
     *  or a <code> NOT_FOUND </code> results. Otherwise, the returned <code> 
     *  Parameter </code> may have a different <code> Id </code> than 
     *  requested, such as the case where a duplicate <code> Id </code> was 
     *  assigned to a <code> Parameter </code> and retained for compatibility. 
     *  
     *  In active mode, parameters are returned that are currently active. In 
     *  any status mode, active and inactive parameters are returned. 
     *
     *  @param  parameterId the <code> Id </code> of the <code> Parameter 
     *          </code> to retrieve 
     *  @return the returned <code> Parameter </code> 
     *  @throws org.osid.NotFoundException no <code> Parameter </code> found 
     *          with the given <code> Id </code> 
     *  @throws org.osid.NullArgumentException <code> parameterId </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.configuration.Parameter getParameter(org.osid.id.Id parameterId)
        throws org.osid.NotFoundException,
               org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a <code> ParameterList </code> corresponding to the given <code> 
     *  IdList. </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all of the 
     *  parameters specified in the <code> Id </code> list, in the order of 
     *  the list, including duplicates, or an error results if an <code> Id 
     *  </code> in the supplied list is not found or inaccessible. Otherwise, 
     *  inaccessible <code> Parameters </code> may be omitted from the list 
     *  and may present the elements in any order including returning a unique 
     *  set. 
     *  
     *  In active mode, parameters are returned that are currently active. In 
     *  any status mode, active and inactive parameters are returned. 
     *
     *  @param  parameterIds the list of <code> Ids </code> to retrieve 
     *  @return the returned <code> Parameter </code> list 
     *  @throws org.osid.NotFoundException an <code> Id was </code> not found 
     *  @throws org.osid.NullArgumentException <code> parameterIds </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.configuration.ParameterList getParametersByIds(org.osid.id.IdList parameterIds)
        throws org.osid.NotFoundException,
               org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a <code> ParameterList </code> corresponding to the given 
     *  parameter genus <code> Type </code> which does not include parameters 
     *  of genus types derived from the specified <code> Type. </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  parameters or an error results. Otherwise, the returned list may 
     *  contain only those parameters that are accessible through this 
     *  session. 
     *  
     *  In active mode, parameters are returned that are currently active. In 
     *  any status mode, active and inactive parameters are returned. 
     *
     *  @param  parameterGenusType a parameter genus type 
     *  @return the returned <code> Parameter list </code> 
     *  @throws org.osid.NullArgumentException <code> parameterGenusType 
     *          </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.configuration.ParameterList getParametersByGenusType(org.osid.type.Type parameterGenusType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a <code> ParameterList </code> corresponding to the given 
     *  parameters genus <code> Type </code> and include any additional 
     *  parameters with genus types derived from the specified <code> Type. 
     *  </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  parameters or an error results. Otherwise, the returned list may 
     *  contain only those parameters that are accessible through this 
     *  session. 
     *  
     *  In active mode, parameters are returned that are currently active. In 
     *  any status mode, active and inactive parameters are returned. 
     *
     *  @param  parameterGenusType a parameter genus type 
     *  @return the returned <code> Parameter list </code> 
     *  @throws org.osid.NullArgumentException <code> parameterGenusType 
     *          </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.configuration.ParameterList getParametersByParentGenusType(org.osid.type.Type parameterGenusType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a <code> ParameterList </code> corresponding to the given 
     *  parameter record <code> Type </code> which does not include parameters 
     *  of record types derived from the specified <code> Type. </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  parameters or an error results. Otherwise, the returned list may 
     *  contain only those parameters that are accessible through this 
     *  session. 
     *  
     *  In active mode, parameters are returned that are currently active. In 
     *  any status mode, active and inactive parameters are returned. 
     *
     *  @param  parameterRecordType a parameter type 
     *  @return the returned <code> Parameter </code> list 
     *  @throws org.osid.NullArgumentException <code> parameterRecordType 
     *          </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.configuration.ParameterList getParametersByRecordType(org.osid.type.Type parameterRecordType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets all <code> Parameters. </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  parameters or an error results. Otherwise, the returned list may 
     *  contain only those parameters that are accessible through this 
     *  session. 
     *  
     *  In active mode, parameters are returned that are currently active. In 
     *  any status mode, active and inactive parameters are returned. 
     *
     *  @return a list of <code> Parameters </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.configuration.ParameterList getParameters()
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;
}
