//
// org.osid.course.ActivityLookupSession
//
//     The OSID Java specification for org.osid.course.ActivityLookupSession.
//
// Copyright (C) 2002-2004, 2007 Massachusetts Institute of Technology. 
// 
// Copyright (C) 2010-2011 Ingenescus. All Rights Reserved. 
//
//     This Work is being provided by the copyright holder(s) subject to the 
//     following license. By obtaining, using and/or copying this Work, you 
//     agree that you have read, understand, and will comply with the 
//     following terms and conditions. 
//     
//     Permission to use, copy and distribute unmodified versions of this 
//     Work, for any purpose, without fee or royalty is hereby granted, 
//     provided that you include the above copyright notices and the terms of 
//     this license on ALL copies of the Work or portions thereof. 
//     
//     You may modify or create Derivatives of this Work only for your 
//     internal purposes. You shall not distribute or transfer any such 
//     Derivative of this Work to any location or to any third party. For the 
//     purposes of this license, "Derivative" shall mean any derivative of the 
//     Work as defined in the United States Copyright Act of 1976, such as a 
//     translation or modification. 
//     
//     This Work and the information contained herein is provided on an "AS 
//     IS" basis WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING 
//     BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
//     PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS 
//     OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
//     LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
//     FROM, OUT OF OR IN CONNECTION WITH THE WORK OR THE USE OR OTHER 
//     DEALINGS IN THE WORK. 
//     
//     The export of software employing encryption technology may require a 
//     specific license from the United States Government. It is the 
//     responsibility of any person or organization contemplating export to 
//     obtain such a license before exporting this Work. 
//

package org.osid.course;


/**
 *  <p>This session provides methods for retrieving <code> Activity </code> 
 *  objects. The <code> Activity </code> represents a time period in which 
 *  courses are offered. </p> 
 *  
 *  <p> This session defines views that offer differing behaviors when 
 *  retrieving multiple objects. </p> 
 *  
 *  <ul>
 *      <li> comparative view: elements may be silently omitted or re-ordered 
 *      </li> 
 *      <li> plenary view: provides a complete set or is an error condition 
 *      </li> 
 *      <li> isolated course catalog view: All activity methods in this 
 *      session operate, retrieve and pertain to activities defined explicitly 
 *      in the current course catalog. Using an isolated view is useful for 
 *      managing <code> Activities </code> with the <code> 
 *      ActivityAdminSession. </code> </li> 
 *      <li> federated course catalog view: All activity methods in this 
 *      session operate, retrieve and pertain to all activities defined in 
 *      this course catalog and any other activities implicitly available in 
 *      this course catalog through course catalog inheritence. </li> 
 *      <li> effective activity view: All course offering lookup methods 
 *      return activities where the current date falls within the effective 
 *      dates inclusive. </li> 
 *      <li> any effective activity view: Activities of any effective or 
 *      expired status are returned from methods. </li> 
 *      <li> denormalized activity view: activities are expanded into a series 
 *      of non-recurring single-date activities </li> 
 *      <li> normalized acticity view: activities are represented by a single 
 *      activity </li> 
 *  </ul>
 *  
 *  
 *  <p> Generally, the comparative view should be used for most applications 
 *  as it permits operation even if there is data that cannot be accessed. The 
 *  methods <code> useFederatedCourseCatalogView() </code> and <code> 
 *  useIsolatedCourseCatalogView() </code> behave as a radio group and one 
 *  should be selected before invoking any lookup methods. </p>
 */

public interface ActivityLookupSession
    extends org.osid.OsidSession {


    /**
     *  Gets the <code> CourseCatalog </code> <code> Id </code> associated 
     *  with this session. 
     *
     *  @return the <code> CourseCatalog Id </code> associated with this 
     *          session 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.id.Id getCourseCatalogId();


    /**
     *  Gets the <code> CourseCatalog </code> associated with this session. 
     *
     *  @return the course catalog 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.course.CourseCatalog getCourseCatalog()
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Tests if this user can perform <code> Activity </code> lookups. A 
     *  return of true does not guarantee successful authorization. A return 
     *  of false indicates that it is known all methods in this session will 
     *  result in a <code> PERMISSION_DENIED. </code> This is intended as a 
     *  hint to an application that may opt not to offer lookup operations to 
     *  unauthorized users. 
     *
     *  @return <code> false </code> if lookup methods are not authorized, 
     *          <code> true </code> otherwise 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean canLookupActivities();


    /**
     *  The returns from the lookup methods may omit or translate elements 
     *  based on this session, such as authorization, and not result in an 
     *  error. This view is used when greater interoperability is desired at 
     *  the expense of precision. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useComparativeActivityView();


    /**
     *  A complete view of the <code> Activity </code> returns is desired. 
     *  Methods will return what is requested or result in an error. This view 
     *  is used when greater precision is desired at the expense of 
     *  interoperability. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void usePlenaryActivityView();


    /**
     *  Federates the view for methods in this session. A federated view will 
     *  include activities in course catalogs which are children of this 
     *  course catalog in the course catalog hierarchy. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useFederatedCourseCatalogView();


    /**
     *  Isolates the view for methods in this session. An isolated view 
     *  restricts lookups to this course catalog only. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useIsolatedCourseCatalogView();


    /**
     *  All activity methods return activities where the current date falls 
     *  within the effective dates inclusive. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useEffectiveActivityView();


    /**
     *  All methods return activities of any effective or expired status. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useAnyEffectiveActivityView();


    /**
     *  A normalized view uses a single <code> Activity </code> to represent a 
     *  set of recurring activities. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useNormalizedActivityView();


    /**
     *  A denormalized view expands recurring activities into a series of 
     *  activities. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useDenormalizedActivityView();


    /**
     *  Gets the <code> Activity </code> specified by its <code> Id. </code> 
     *  In plenary mode, the exact <code> Id </code> is found or a <code> 
     *  NOT_FOUND </code> results. Otherwise, the returned <code> Activity 
     *  </code> may have a different <code> Id </code> than requested, such as 
     *  the case where a duplicate <code> Id </code> was assigned to an <code> 
     *  Activity </code> and retained for compatibility. 
     *  
     *  In effective mode, activities are returned where the current date 
     *  falls within the effective dates inclusive. In any effective mode, 
     *  effective and expired activities are returned. 
     *
     *  @param  activityId <code> Id </code> of the <code> Activity </code> 
     *  @return the activity 
     *  @throws org.osid.NotFoundException <code> activityId </code> not found 
     *  @throws org.osid.NullArgumentException <code> activityId </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method is must be implemented. 
     */

    public org.osid.course.Activity getActivity(org.osid.id.Id activityId)
        throws org.osid.NotFoundException,
               org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> ActivityList </code> corresponding to the given <code> 
     *  IdList. </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all of the 
     *  activities specified in the <code> Id </code> list, in the order of 
     *  the list, including duplicates, or an error results if an <code> Id 
     *  </code> in the supplied list is not found or inaccessible. Otherwise, 
     *  inaccessible <code> Activities </code> may be omitted from the list 
     *  and may present the elements in any order including returning a unique 
     *  set. 
     *  
     *  In effective mode, activities are returned where the current date 
     *  falls within the effective dates inclusive. In any effective mode, 
     *  effective and expired activities are returned. 
     *
     *  @param  activityIds the list of <code> Ids </code> to retrieve 
     *  @return the returned <code> Activity </code> list 
     *  @throws org.osid.NotFoundException an <code> Id was </code> not found 
     *  @throws org.osid.NullArgumentException <code> activityIds </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.course.ActivityList getActivitiesByIds(org.osid.id.IdList activityIds)
        throws org.osid.NotFoundException,
               org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> ActivityList </code> corresponding to the given 
     *  activity genus <code> Type </code> which does not include activities 
     *  of genus types derived from the specified <code> Type. </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  activities or an error results. Otherwise, the returned list may 
     *  contain only those activities that are accessible through this 
     *  session. 
     *  
     *  In effective mode, activities are returned where the current date 
     *  falls within the effective dates inclusive. In any effective mode, 
     *  effective and expired activities are returned. 
     *
     *  @param  activityGenusType an activity genus type 
     *  @return the returned <code> Activity </code> list 
     *  @throws org.osid.NullArgumentException <code> activityGenusType 
     *          </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.course.ActivityList getActivitiesByGenusType(org.osid.type.Type activityGenusType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> ActivityList </code> corresponding to the given 
     *  activity genus <code> Type </code> and include any additional 
     *  activities with genus types derived from the specified <code> Type. 
     *  </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  activities or an error results. Otherwise, the returned list may 
     *  contain only those activities that are accessible through this 
     *  session. 
     *  
     *  In effective mode, activities are returned where the current date 
     *  falls within the effective dates inclusive. In any effective mode, 
     *  effective and expired activities are returned. 
     *
     *  @param  activityGenusType an activity genus type 
     *  @return the returned <code> Activity </code> list 
     *  @throws org.osid.NullArgumentException <code> activityGenusType 
     *          </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.course.ActivityList getActivitiesByParentGenusType(org.osid.type.Type activityGenusType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> ActivityList </code> containing the given activity 
     *  record <code> Type. </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  activities or an error results. Otherwise, the returned list may 
     *  contain only those activities that are accessible through this 
     *  session. 
     *  
     *  In effective mode, activities are returned where the current date 
     *  falls within the effective dates inclusive. In any effective mode, 
     *  effective and expired activities are returned. 
     *
     *  @param  activityRecordType an activity record type 
     *  @return the returned <code> Activity </code> list 
     *  @throws org.osid.NullArgumentException <code> activityRecordType 
     *          </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.course.ActivityList getActivitiesByRecordType(org.osid.type.Type activityRecordType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> ActivityList </code> effective within the given date 
     *  range inclusive. <code> </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  activities or an error results. Otherwise, the returned list may 
     *  contain only those activities that are accessible through this 
     *  session. 
     *  
     *  In effective mode, activities are returned where the current date 
     *  falls within the effective dates inclusive. In any effective mode, 
     *  effective and expired activities are returned. 
     *
     *  @param  from start of date range 
     *  @param  to end of date range 
     *  @return the returned <code> Activity </code> list 
     *  @throws org.osid.InvalidArgumentException <code> from </code> is 
     *          greater than <code> to </code> 
     *  @throws org.osid.NullArgumentException <code> from </code> or <code> 
     *          to </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.course.ActivityList getActivitiesOnDate(org.osid.calendaring.DateTime from, 
                                                            org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets all <code> Activities </code> associated with a given <code> 
     *  ActivityUnit. </code> 
     *  
     *  In plenary mode, the returned list contains all known activities or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  activities that are accessible through this session. 
     *  
     *  In effective mode, activities are returned where the current date 
     *  falls within the effective dates inclusive. In any effective mode, 
     *  effective and expired activities are returned. 
     *
     *  @param  activityUnitId an activity unit <code> Id </code> 
     *  @return the returned <code> Activity </code> list 
     *  @throws org.osid.NullArgumentException <code> activityUnitId </code> 
     *          is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.course.ActivityList getActivitiesForActivityUnit(org.osid.id.Id activityUnitId)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> ActivityList </code> for the given activity unit and 
     *  effective within the given date range inclusive. <code> </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  activities or an error results. Otherwise, the returned list may 
     *  contain only those activities that are accessible through this 
     *  session. 
     *  
     *  In effective mode, activities are returned where the current date 
     *  falls within the effective dates inclusive. In any effective mode, 
     *  effective and expired activities are returned. 
     *
     *  @param  activityUnitId an activity unit <code> Id </code> 
     *  @param  from start of date range 
     *  @param  to end of date range 
     *  @return the returned <code> Activity </code> list 
     *  @throws org.osid.InvalidArgumentException <code> from </code> is 
     *          greater than <code> to </code> 
     *  @throws org.osid.NullArgumentException <code> activityUnitId, from, 
     *          </code> or <code> to </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.course.ActivityList getActivitiesForActivityUnitOnDate(org.osid.id.Id activityUnitId, 
                                                                           org.osid.calendaring.DateTime from, 
                                                                           org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets all <code> Activities </code> associated with a given term. 
     *  
     *  In plenary mode, the returned list contains all known activities or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  activities that are accessible through this session. 
     *  
     *  In effective mode, activities are returned where the current date 
     *  falls within the effective dates inclusive. In any effective mode, 
     *  effective and expired activities are returned. 
     *
     *  @param  termId a term <code> Id </code> 
     *  @return the returned <code> Activity </code> list 
     *  @throws org.osid.NullArgumentException <code> termId </code> is <code> 
     *          null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.course.ActivityList getActivitiesForTerm(org.osid.id.Id termId)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> ActivityList </code> for the given term and effective 
     *  within the given date range inclusive. <code> </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  activities or an error results. Otherwise, the returned list may 
     *  contain only those activities that are accessible through this 
     *  session. 
     *  
     *  In effective mode, activities are returned where the current date 
     *  falls within the effective dates inclusive. In any effective mode, 
     *  effective and expired activities are returned. 
     *
     *  @param  termId a term <code> Id </code> 
     *  @param  from start of date range 
     *  @param  to end of date range 
     *  @return the returned <code> Activity </code> list 
     *  @throws org.osid.InvalidArgumentException <code> from </code> is 
     *          greater than <code> to </code> 
     *  @throws org.osid.NullArgumentException <code> termId, from, </code> or 
     *          <code> to </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.course.ActivityList getActivitiesForTermOnDate(org.osid.id.Id termId, 
                                                                   org.osid.calendaring.DateTime from, 
                                                                   org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets all <code> Activities </code> associated with a given <code> 
     *  ActivityUnit </code> for a term. 
     *  
     *  In plenary mode, the returned list contains all known activities or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  activities that are accessible through this session. 
     *  
     *  In effective mode, activities are returned where the current date 
     *  falls within the effective dates inclusive. In any effective mode, 
     *  effective and expired activities are returned. 
     *
     *  @param  activityUnitId an activity unit <code> Id </code> 
     *  @param  termId a term <code> Id </code> 
     *  @return the returned <code> Activity </code> list 
     *  @throws org.osid.NullArgumentException <code> activityUnitId </code> 
     *          or <code> termId </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.course.ActivityList getActivitiesForActivityUnitAndTerm(org.osid.id.Id activityUnitId, 
                                                                            org.osid.id.Id termId)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> ActivityList </code> for the given activity unit, term, 
     *  and effective within the given date range inclusive. <code> </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  activities or an error results. Otherwise, the returned list may 
     *  contain only those activities that are accessible through this 
     *  session. 
     *  
     *  In effective mode, activities are returned where the current date 
     *  falls within the effective dates inclusive. In any effective mode, 
     *  effective and expired activities are returned. 
     *
     *  @param  activityUnitId an activity unit <code> Id </code> 
     *  @param  termId a term <code> Id </code> 
     *  @param  from start of date range 
     *  @param  to end of date range 
     *  @return the returned <code> Activity </code> list 
     *  @throws org.osid.InvalidArgumentException <code> from </code> is 
     *          greater than <code> to </code> 
     *  @throws org.osid.NullArgumentException <code> activityUnitId, termId, 
     *          from, </code> or <code> to </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.course.ActivityList getActivitiesForActivityUnitAndTermOnDate(org.osid.id.Id activityUnitId, 
                                                                                  org.osid.id.Id termId, 
                                                                                  org.osid.calendaring.DateTime from, 
                                                                                  org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets all <code> Activities </code> associated with a given <code> 
     *  CourseOffering. </code> 
     *  
     *  In plenary mode, the returned list contains all known activities or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  activities that are accessible through this session. 
     *  
     *  In effective mode, activities are returned where the current date 
     *  falls within the effective dates inclusive. In any effective mode, 
     *  effective and expired activities are returned. 
     *
     *  @param  courseOfferingId a course <code> Id </code> 
     *  @return the returned <code> Activity </code> list 
     *  @throws org.osid.NullArgumentException <code> courseOfferingId </code> 
     *          is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.course.ActivityList getActivitiesForCourseOffering(org.osid.id.Id courseOfferingId)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets all <code> Activities. </code> 
     *  
     *  In plenary mode, the returned list contains all known activities or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  activities that are accessible through this session. 
     *  
     *  In effective mode, activities are returned where the current date 
     *  falls within the effective dates inclusive. In any effective mode, 
     *  effective and expired activities are returned. 
     *
     *  @return a list of <code> Activities </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.course.ActivityList getActivities()
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;
}
