//
// OsidParameter.java
//
//     The definition for an OSID Parameter.
//
//
// Tom Coppeto
// OnTapSolutions
// 4 January 2009
//
// Copyright (c) 2009 Massachusetts Institute of Technology. All Rights
// Reserved.
//
//      Permission is hereby granted, free of charge, to any person
//      obtaining a copy of this software and associated documentation
//      files (the "Software"), to deal in the Software without
//      restriction, including without limitation the rights to use,
//      copy, modify, merge, publish, distribute, sublicesne, and/or
//      sell copies of the Software, and to permit the persons to whom the
//      Software is furnished to do so, subject the following conditions:
//
//      The above copyright notice and this permission notice shall be
//      included in all copies or substantial portions of the Software.
//
//      The Software is provided "AS IS", WITHOUT WARRANTY OF ANY KIND,
//      EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
//      OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
//      NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
//      HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
//      WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//      OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
//      DEALINGS IN THE SOFTWARE.
//

package org.osid.meta;


/**
 *  The definition for an OSID parameter.
 */

public class OsidParameter {
    private final String name;
    private final String type;
    private final String specType;
    private final boolean array;
    private final boolean primitive;
    private final boolean enumeration;


    /**
     *  Constructs a new <code>OsidParameter</code> for
     *  enumerations. This constructor marks the parameter as an
     *  enumeration.
     *
     *  @param name the name of the enumeration
     *  @param type the type of the enumeration
     */

    protected OsidParameter(String name, String type) {
        this.name        = name;
        this.type        = type;
        this.array       = false;
        this.specType    = type;
        this.primitive   = false;
        this.enumeration = true;

        return;
    }


    /**
     *  Constructs a new <code>OsidParameter</code> for non-primitives.
     *
     *  @param name the parameter name
     *  @param type the name of the primitive or interface
     *  @param array <code>true</code> if the parameter is an array
     *         <code>false</code> otherwise
     */

    protected OsidParameter(String name, String type, boolean array) {
        this.name        = name;
        this.type        = type;
        this.array       = array;
        this.specType    = type;
        this.primitive   = false;
        this.enumeration = false;

        return;
    }


    /**
     *  Constructs a new <code>OsidParameter</code> for
     *  primitives. This constructor marks the parameter as a
     *  primitive.
     *
     *  @param name the parameter name
     *  @param type the name of the primitive or interface
     *  @param array <code>true</code> if the parameter is an array
     *         <code>false</code> otherwise
     *  @param specPrimitiveType the primitive type according to spec 
     */

    protected OsidParameter(String name, String type, boolean array, String specPrimitiveType) {
        this.name        = name;
        this.type        = type;
        this.array       = array;
        this.specType    = specPrimitiveType;
        this.primitive   = true;
        this.enumeration = false;

        return;
    }


    /**
     *  Gets the name of this parameter.
     *
     *  @return the parameter name
     */

    public String getName() {
        return (this.name);
    }


    /**
     *  Gets the type of this parameter.
     *
     *  @return the parameter type
     */

    public String getType() {
        return (this.type);
    }


    /**
     *  Gets the specification type of this parameter. 
     *
     *  @return the specification type
     */

    public String getSpecType() {
        return (this.specType);
    }


    /**
     *  Tests if this parameter is an OSID primitive.
     *
     *  @return <code>true</code> if this parameter is an OSID
     *          primitive, <code>code</code> otherwise
     */

    public boolean isPrimitive() {
        return (this.primitive);
    }


    /**
     *  Tests if this parameter is an OSID enumeration.
     *
     *  @return <code>true</code> if this parameter is an OSID
     *          enumeration, <code>code</code> otherwise
     */

    public boolean isEnumeration() {
        return (this.enumeration);
    }


    /**
     *  Tests if this parameter is an array.
     *
     *  @return <code>true</code> if this parameter is an array,
     *          <code>code</code> otherwise
     */

    public boolean isArray() {
        return (this.array);
    }
}