//
// org.osid.offering.ResultLookupSession
//
//     The OSID Java specification for org.osid.offering.ResultLookupSession.
//
// Copyright (C) 2013 Ingenescus. All Rights Reserved. 
//
//     This Work is being provided by the copyright holder(s) subject to the 
//     following license. By obtaining, using and/or copying this Work, you 
//     agree that you have read, understand, and will comply with the 
//     following terms and conditions. 
//     
//     Permission to use, copy and distribute unmodified versions of this 
//     Work, for any purpose, without fee or royalty is hereby granted, 
//     provided that you include the above copyright notices and the terms of 
//     this license on ALL copies of the Work or portions thereof. 
//     
//     You may modify or create Derivatives of this Work only for your 
//     internal purposes. You shall not distribute or transfer any such 
//     Derivative of this Work to any location or to any third party. For the 
//     purposes of this license, "Derivative" shall mean any derivative of the 
//     Work as defined in the United States Copyright Act of 1976, such as a 
//     translation or modification. 
//     
//     This Work and the information contained herein is provided on an "AS 
//     IS" basis WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING 
//     BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
//     PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS 
//     OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
//     LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
//     FROM, OUT OF OR IN CONNECTION WITH THE WORK OR THE USE OR OTHER 
//     DEALINGS IN THE WORK. 
//     
//     The export of software employing encryption technology may require a 
//     specific license from the United States Government. It is the 
//     responsibility of any person or organization contemplating export to 
//     obtain such a license before exporting this Work. 
//

package org.osid.offering;


/**
 *  <p>This session defines methods for retrieving results. </p> 
 *  
 *  <p> This lookup session defines several views: </p> 
 *  
 *  <ul>
 *      <li> comparative view: elements may be silently omitted or re-ordered 
 *      </li> 
 *      <li> plenary view: provides a complete result set or is an error 
 *      condition </li> 
 *      <li> isolated catalogue view: All result methods in this session 
 *      operate, retrieve and pertain to results defined explicitly in the 
 *      current catalogue. Using an isolated view is useful for managing 
 *      results with the <code> ResultAdminSession. </code> </li> 
 *      <li> federated catalogue view: All result lookup methods in this 
 *      session operate, retrieve and pertain to all results defined in this 
 *      catalogue and any other catalogues implicitly available in this 
 *      catalogue through catalogue inheritence. </li> 
 *      <li> effective result view: All result lookup methods return results 
 *      where the current date falls in between the effective dates inclusive. 
 *      </li> 
 *      <li> any effective result view: Results of any effective date are 
 *      returned from methods. </li> 
 *  </ul>
 *  
 *  
 *  <p> The methods <code> useFederatedCatalogueView() </code> and <code> 
 *  useIsolatedCatalogueView() </code> behave as a radio group and one should 
 *  be selected before invoking any lookup methods. </p>
 */

public interface ResultLookupSession
    extends org.osid.OsidSession {


    /**
     *  Gets the <code> Catalogue </code> <code> Id </code> associated with 
     *  this session. 
     *
     *  @return the <code> Catalogue Id </code> associated with this session 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.id.Id getCatalogueId();


    /**
     *  Gets the <code> Catalogue </code> associated with this session. 
     *
     *  @return the catalogue 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.offering.Catalogue getCatalogue()
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Tests if this user can examine this catalogue. A return of true does 
     *  not guarantee successful authorization. A return of false indicates 
     *  that it is known all methods in this session will result in a <code> 
     *  PERMISSION_DENIED. </code> This is intended as a hint to an 
     *  application that may opt not to offer these operations. 
     *
     *  @return <code> false </code> if catalogue reading methods are not 
     *          authorized, <code> true </code> otherwise 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean canLookupResults();


    /**
     *  The returns from the lookup methods may omit or translate elements 
     *  based on this session, such as authorization, and not result in an 
     *  error. This view is used when greater interoperability is desired at 
     *  the expense of precision. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useComparativeResultView();


    /**
     *  A complete view of the <code> Result </code> returns is desired. 
     *  Methods will return what is requested or result in an error. This view 
     *  is used when greater precision is desired at the expense of 
     *  interoperability. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void usePlenaryResultView();


    /**
     *  Federates the view for methods in this session. A federated view will 
     *  include results in catalogues which are children of this catalogue in 
     *  the catalogue hierarchy. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useFederatedCatalogueView();


    /**
     *  Isolates the view for methods in this session. An isolated view 
     *  restricts retrievals to this catalogue only. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useIsolatedCatalogueView();


    /**
     *  Only results whose effective dates are current are returned by methods 
     *  in this session. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useEffectiveResultView();


    /**
     *  All results of any effective dates are returned by methods in this 
     *  session. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useAnyEffectiveResultView();


    /**
     *  Gets the <code> Result </code> specified by its <code> Id. </code> 
     *  
     *  <code> </code> In plenary mode, the exact <code> Id </code> is found 
     *  or a <code> NOT_FOUND </code> results. Otherwise, the returned <code> 
     *  Result </code> may have a different <code> Id </code> than requested, 
     *  such as the case where a duplicate <code> Id </code> was assigned to 
     *  an <code> Result </code> and retained for compatibility. 
     *  
     *  In effective mode, results are returned that are currently effective. 
     *  In any effective mode, effective results and those currently expired 
     *  are returned. 
     *
     *  @param  resultId the <code> Id </code> of the <code> Result </code> to 
     *          retrieve 
     *  @return the returned <code> Result </code> 
     *  @throws org.osid.NotFoundException no <code> Result </code> found with 
     *          the given <code> Id </code> 
     *  @throws org.osid.NullArgumentException <code> resultId </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.offering.Result getResult(org.osid.id.Id resultId)
        throws org.osid.NotFoundException,
               org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> ResultList </code> corresponding to the given <code> 
     *  IdList. </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all of the 
     *  results specified in the <code> Id </code> list, in the order of the 
     *  list, including duplicates, or an error results if an <code> Id 
     *  </code> in the supplied list is not found or inaccessible. Otherwise, 
     *  inaccessible results may be omitted from the list and may present the 
     *  elements in any order including returning a unique set. 
     *  
     *  In effective mode, results are returned that are currently effective. 
     *  In any effective mode, effective results and those currently expired 
     *  are returned. 
     *
     *  @param  resultIds the list of <code> Ids </code> to retrieve 
     *  @return the returned <code> Result list </code> 
     *  @throws org.osid.NotFoundException an <code> Id was </code> not found 
     *  @throws org.osid.NullArgumentException <code> resultIds </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.offering.ResultList getResultsByIds(org.osid.id.IdList resultIds)
        throws org.osid.NotFoundException,
               org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> ResultList </code> corresponding to the given result 
     *  genus <code> Type </code> which does not include results of types 
     *  derived from the specified <code> Type. </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  results or an error results. Otherwise, the returned list may contain 
     *  only those results that are accessible through this session. 
     *  
     *  In effective mode, results are returned that are currently effective. 
     *  In any effective mode, effective results and those currently expired 
     *  are returned. 
     *
     *  @param  resultGenusType an results genus type 
     *  @return the returned <code> Result list </code> 
     *  @throws org.osid.NullArgumentException <code> resultGenusType </code> 
     *          is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.offering.ResultList getResultsByGenusType(org.osid.type.Type resultGenusType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> ResultList </code> corresponding to the given result 
     *  genus <code> Type </code> and include any additional results with 
     *  genus types derived from the specified <code> Type. </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  results or an error results. Otherwise, the returned list may contain 
     *  only those results that are accessible through this session. 
     *  
     *  In effective mode, results are returned that are currently effective. 
     *  In any effective mode, effective results and those currently expired 
     *  are returned. 
     *
     *  @param  resultGenusType a result genus type 
     *  @return the returned <code> Result list </code> 
     *  @throws org.osid.NullArgumentException <code> resultGenusType </code> 
     *          is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.offering.ResultList getResultsByParentGenusType(org.osid.type.Type resultGenusType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> ResultList </code> containing the given result record 
     *  <code> Type. </code> 
     *  
     *  In plenary mode, the returned list contains all known results or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  results that are accessible through this session. 
     *  
     *  In effective mode, results are returned that are currently effective. 
     *  In any effective mode, effective results and those currently expired 
     *  are returned. 
     *
     *  @param  resultRecordType an result record type 
     *  @return the returned <code> ResultList </code> 
     *  @throws org.osid.NullArgumentException <code> resultRecordType </code> 
     *          is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.offering.ResultList getResultsByRecordType(org.osid.type.Type resultRecordType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> ResultList </code> effective during the entire given 
     *  date range inclusive but not confined to the date range. 
     *  
     *  In plenary mode, the returned list contains all known results or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  results that are accessible through this session. 
     *  
     *  In effective mode, results are returned that are currently effective. 
     *  In any effective mode, effective results and those currently expired 
     *  are returned. 
     *
     *  @param  from start of date range 
     *  @param  to end of date range 
     *  @return the returned <code> ResultList </code> 
     *  @throws org.osid.InvalidArgumentException <code> from </code> is 
     *          greater than <code> to </code> 
     *  @throws org.osid.NullArgumentException <code> from or to </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.offering.ResultList getResultsOnDate(org.osid.calendaring.DateTime from, 
                                                         org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> ResultList </code> by genus type effective during the 
     *  entire given date range inclusive but not confined to the date range. 
     *  
     *  In plenary mode, the returned list contains all known results or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  results that are accessible through this session. 
     *  
     *  In effective mode, results are returned that are currently effective. 
     *  In any effective mode, effective results and those currently expired 
     *  are returned. 
     *
     *  @param  resultGenusType a results genus type 
     *  @param  from start of date range 
     *  @param  to end of date range 
     *  @return the returned <code> ResultList </code> 
     *  @throws org.osid.InvalidArgumentException <code> from </code> is 
     *          greater than <code> to </code> 
     *  @throws org.osid.NullArgumentException <code> resultGenusType, from or 
     *          to </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.offering.ResultList getResultsByGenusTypeOnDate(org.osid.type.Type resultGenusType, 
                                                                    org.osid.calendaring.DateTime from, 
                                                                    org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> ResultList </code> for the given participant <code> Id. 
     *  </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all of the 
     *  results corresponding to the given participant, including duplicates, 
     *  or an error results if an result is inaccessible. Otherwise, 
     *  inaccessible <code> Results </code> may be omitted from the list. 
     *  
     *  In effective mode, results are returned that are currently effective. 
     *  In any effective mode, effective results and those currently expired 
     *  are returned. 
     *
     *  @param  participantId a participant <code> Id </code> 
     *  @return the returned <code> ResultList </code> 
     *  @throws org.osid.NullArgumentException <code> participantId </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.offering.ResultList getResultsForParticipant(org.osid.id.Id participantId)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> ResultList </code> for the given participant <code> Id. 
     *  </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all of the 
     *  results corresponding to the given participant, including duplicates, 
     *  or an error results if an result is inaccessible. Otherwise, 
     *  inaccessible <code> Results </code> may be omitted from the list. 
     *  
     *  In effective mode, results are returned that are currently effective. 
     *  In any effective mode, effective results and those currently expired 
     *  are returned. 
     *
     *  @param  participantId a participant <code> Id </code> 
     *  @param  resultGenusType a results genus type 
     *  @return the returned <code> ResultList </code> 
     *  @throws org.osid.NullArgumentException <code> participantId </code> or 
     *          <code> resultGenusType </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.offering.ResultList getResultsByGenusTypeForParticipant(org.osid.id.Id participantId, 
                                                                            org.osid.type.Type resultGenusType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of results for a participant and effective during the 
     *  entire given date range inclusive but not confined to the date range. 
     *  
     *  In plenary mode, the returned list contains all known results or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  results that are accessible through this session. 
     *  
     *  In effective mode, results are returned that are currently effective. 
     *  In any effective mode, effective results and those currently expired 
     *  are returned. 
     *
     *  @param  participantId a participant <code> Id </code> 
     *  @param  from start of date range 
     *  @param  to end of date range 
     *  @return the returned <code> ResultList </code> 
     *  @throws org.osid.InvalidArgumentException <code> from </code> is 
     *          greater than <code> to </code> 
     *  @throws org.osid.NullArgumentException <code> participantId, from, 
     *          </code> or <code> to </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.offering.ResultList getResultsForParticipantOnDate(org.osid.id.Id participantId, 
                                                                       org.osid.calendaring.DateTime from, 
                                                                       org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of results for a participant and effective during the 
     *  entire given date range inclusive but not confined to the date range. 
     *  
     *  In plenary mode, the returned list contains all known results or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  results that are accessible through this session. 
     *  
     *  In effective mode, results are returned that are currently effective. 
     *  In any effective mode, effective results and those currently expired 
     *  are returned. 
     *
     *  @param  participantId a participant <code> Id </code> 
     *  @param  resultGenusType a results genus type 
     *  @param  from start of date range 
     *  @param  to end of date range 
     *  @return the returned <code> ResultList </code> 
     *  @throws org.osid.InvalidArgumentException <code> from </code> is 
     *          greater than <code> to </code> 
     *  @throws org.osid.NullArgumentException <code> participantId, 
     *          resultGenusType, from, </code> or <code> to </code> is <code> 
     *          null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.offering.ResultList getResultsByGenusTypeForParticipantOnDate(org.osid.id.Id participantId, 
                                                                                  org.osid.type.Type resultGenusType, 
                                                                                  org.osid.calendaring.DateTime from, 
                                                                                  org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets all results. 
     *  
     *  In plenary mode, the returned list contains all known results or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  results that are accessible through this session. 
     *  
     *  In effective mode, results are returned that are currently effective. 
     *  In any effective mode, effective results and those currently expired 
     *  are returned. 
     *
     *  @return a list of results 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.offering.ResultList getResults()
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;
}
