/**
 * <p>The Open Service Interface Definitions for the org.osid
 * service.</p>
 * 
 * <p>The Open Service Interface Definitions (OSIDs) is a service-based 
 * architecture to promote software interoperability. The OSIDs are a large 
 * suite of interface contract specifications that describe the integration 
 * points among services and system components for the purpose of creating 
 * choice among a variety of different and independently developed 
 * applications and systems, allowing independent evolution of software 
 * components within a complex system, and federated service providers. </p> 
 * 
 * <p> The OSIDs were initially developed in 2001 as part of the MIT Open 
 * Knowledge Initiative Project funded by the Andrew W. Mellon Foundation to 
 * provide an architecture for higher education learning systems. OSID 3K 
 * development began in 2006 to redesign the capabilities of the 
 * specifications to apply to a much broader range of service domains and 
 * integration challenges among both small and large-scale enterprise systems. 
 * </p> 
 * 
 * <p> The <code> osid </code> package defines the building blocks for the 
 * OSIDs which are defined in packages for their respective services. This 
 * package defines the top-level interfaces used by all the OSIDs as well as 
 * specification metadata and the OSID Runtime interface. </p> 
 * <h4> Meta Interfaces and Enumerations </h4> 
 * 
 * <ul>
 *     <li> <code> OSID: </code> an enumeration listing the OSIDs defined in 
 *     the specification. </li> 
 *     <li> <code> Syntax: </code> an enumeration listing primitive types 
 *     </li> 
 *     <li> <code> Metadata: </code> an interface for describing data 
 *     constraints on a data element </li> 
 * </ul>
 * 
 * <h4> Interface Behavioral Markers </h4> 
 * 
 * <p> Interface behavioral markers are used to tag a behavioral pattern of 
 * the interface used to construct other object interfaces. </p> 
 * 
 * <ul>
 *     <li> <code> OsidPrimitive: </code> marks an OSID interface used as a 
 *     primitive. OSID primitives may take the form interfaces if not bound to 
 *     a language primitive. Interfaces used as primitives are marked to 
 *     indicate that the underlying objects may be constructed by an OSID 
 *     Consumer and an OSID Provider must honor any OSID primitive regardless 
 *     of its origin. </li> 
 *     <li> <code> Identifiable: </code> Marks an interface identifiable by an 
 *     OSID <code> Id. </code> </li> 
 *     <li> <code> Extensible: </code> Marks an interface as extensible 
 *     through <code> OsidRecords. </code> </li> 
 *     <li> <code> Browsable: </code> Marks an interface as providing <code> 
 *     Property </code> inspection for its <code> OsidRecords. </code> </li> 
 *     <li> <code> Suppliable: </code> Marks an interface as accepting data 
 *     from an OSID Consumer. </li> 
 *     <li> <code> Temporal: </code> Marks an interface that has a lifetime 
 *     with begin an end dates. </li> 
 *     <li> <code> Subjugateable: </code> Mars an interface that is dependent 
 *     on another object. </li> 
 *     <li> <code> Aggregateable: </code> Marks an interface that contains 
 *     other objects normally related through other services. </li> 
 *     <li> <code> Containable: </code> Marks an interface that contains a 
 *     recursive reference to itself. </li> 
 *     <li> <code> Sourceable: </code> Marks an interface as having a 
 *     provider. </li> 
 *     <li> <code> Federateable: </code> Marks an interface that can be 
 *     federated using the OSID Hierarchy pattern. </li> 
 *     <li> <code> Operable: </code> Marks an interface as responsible for 
 *     performing operatons or tasks. <code> Operables </code> may be enabled 
 *     or disabled. </li> 
 * </ul>
 * 
 * <h4> Abstract service Interfaces </h4> 
 * 
 * <ul>
 *     <li> <code> OsidProfile: </code> Defines interoperability methods used 
 *     by OsidManagers. </li> 
 *     <li> <code> OsidManager: </code> The entry point into an OSID and 
 *     provides access to <code> OsidSessions. </code> </li> 
 *     <li> <code> OsidProxyManager: </code> Another entry point into an OSID 
 *     providing a means for proxying data from a middle tier application 
 *     server to an underlying OSID Provider. </li> 
 *     <li> <code> OsidSession </code> : A service interface accessible from 
 *     an <code> OsidManager </code> that defines a set of methods for an 
 *     aspect of a service. </li> 
 * </ul>
 * 
 * 
 * <p> Object-like interfaces are generally defined along lines of 
 * interoperability separating issues of data access from data management and 
 * searching. These interfaces may also implement any of the abstract 
 * behavioral interfaces listed above. The OSIDs do not adhere to a DAO/DTO 
 * model in its service definitions in that there are service methods defined 
 * on the objects (although they can be implemented using DTOs if desired). 
 * For the sake of an outline, we'll pretend they are data objects. </p> 
 * 
 * <ul>
 *     <li> <code> OsidObject: </code> Defines object data. <code> OsidObjects 
 *     </code> are accessed from <code> OsidSessions. </code> <code> 
 *     OsidObjects </code> are part of an interface hierarchy whose interfaces 
 *     include the behavioral markers and a variety of common <code> 
 *     OsidObjects. </code> All <code> OsidObjects </code> are <code> 
 *     Identifiable, </code> <code> Extensible, </code> and have a <code> 
 *     Type. </code> There are several variants of <code> OsidObjects </code> 
 *     that indicate a more precise behavior. </li> 
 *     <li> <code> OsidObjectQuery: </code> Defines a set of methods to query 
 *     an OSID for its <code> OsidObjects </code> . An <code> OsidQuery 
 *     </code> is accessed from an <code> OsidSession. </code> </li> 
 *     <li> <code> OsidObjectQueryInspector: </code> Defines a set of methods 
 *     to examine an <code> OsidQuery. </code> </li> 
 *     <li> <code> OsidObjectForm: </code> Defines a set of methods to create 
 *     and update data. <code> OsidForms </code> are accessed from <code> 
 *     OsidSessions. </code> </li> 
 *     <li> <code> OsidObjectSearchOrder: </code> Defines a set of methods to 
 *     order search results. <code> OsidSearchOrders </code> are accessed from 
 *     <code> OsidSessions. </code> </li> 
 * </ul>
 * 
 * 
 * <p> Most objects are or are derived from <code> OsidObjects. </code> Some 
 * object interfaces may not implement <code> OsidObejct </code> but instead 
 * derive directly from interface behavioral markers. Other <code> OsidObjects 
 * </code> may include interface behavioral markers to indicate functionality 
 * beyond a plain object. Several categories of <code> OsidObjects </code> 
 * have been defined to cluster behaviors to semantically distinguish their 
 * function in the OSIDs. </p> 
 * 
 * <ul>
 *     <li> <code> OsidCatalog: </code> At the basic level, a catalog 
 *     represents a collection of other <code> OsidObjects. </code> The 
 *     collection may be physical or virtual and may be federated to build 
 *     larger <code> OsidCatalogs </code> using hierarchy services. <code> 
 *     OsidCatalogs </code> may serve as a control point to filter or 
 *     constrain the <code> OsidObjects </code> that may be visible or 
 *     created. Each <code> OsidCatalog </code> may have its own provider 
 *     identifty apart from the service provider. </li> 
 *     <li> <code> OsidRelationship: </code> Relates two <code> OsidObjects. 
 *     </code> The <code> OsidRelationship </code> represents the edge in a 
 *     graph that may have its own relationship type and data. <code> 
 *     OsidRelationships </code> are <code> Temporal </code> in that they have 
 *     a time in which the relationship came into being and a time when the 
 *     relationship ends. </li> 
 *     <li> <code> OsidRule: </code> Defines an injection point for logic. An 
 *     <code> OsidRule </code> may represent some constraint, evaluation, or 
 *     execution. While authoring of <code> OsidRules </code> is outside the 
 *     scope of the OSIDs, an <code> OsidRule </code> provides the mean to 
 *     identify the rule and map it to certain <code> OsidObjects </code> to 
 *     effect behavior of a service. </li> 
 * </ul>
 * 
 * 
 * <p> The most basic operations of an OSID center on retrieval, search, 
 * create &amp; update, and notifications on changes to an <code> OsidObject. 
 * </code> The more advanced OSIDs model a system behavior where a variety of 
 * implicit relationships, constraints and rules come into play. </p> 
 * 
 * <ul>
 *     <li> <code> OsidGovernator: </code> Implies an activity or operation 
 *     exists in the OSID Provider acting as an <code> Operable </code> point 
 *     for a set of rules governing related <code> OsidObjects. </code> The 
 *     <code> OsidGovernator </code> represents an engine of sorts in an OSID 
 *     Provider and may have its own provider identity. </li> 
 *     <li> <code> OsidCompendium </code> : <code> OsidObjects </code> which 
 *     are reports or summaries based on transactional data managed elsewhere. 
 *     </li> 
 * </ul>
 * 
 * 
 * <p> Managing data governing rules occurs in a separate set of interfaces 
 * from the effected <code> OsidObjects </code> (and often in a separate 
 * package). This allows for a normalized set of rules managing a small set of 
 * control points in a potentially large service. </p> 
 * 
 * <ul>
 *     <li> <code> OsidEnabler: </code> A managed control point to enable or 
 *     disable the operation or effectiveness of another <code> OsidObject 
 *     </code> . Enablers create a dynamic environment where behaviors and 
 *     relationships can come and go based on rule evauations. </li> 
 *     <li> <code> OsidConstrainer: </code> A managed control point to 
 *     configure the constraints on the behavior of another <code> OsidObject. 
 *     </code> </li> 
 *     <li> <code> OsidProcessor: </code> A managed control point to configure 
 *     the behavior of another <code> OsidObject </code> where some kins of 
 *     processing is implied. </li> 
 * </ul>
 * 
 * <h4> Other Abstract Interfaces </h4> 
 * 
 * <ul>
 *     <li> <code> OsidSearch: </code> Defines set of methods to manage search 
 *     options for performing searches. </li> 
 *     <li> <code> OsidSearchResults: </code> Defines a set of methods to 
 *     examine search results. </li> 
 * </ul>
 * 
 * 
 * <ul>
 *     <li> <code> OsidReceiver: </code> Defines a set of methods invoked for 
 *     asynchronous notification. </li> 
 *     <li> <code> OsidList: </code> Defines a set of methods to sequentially 
 *     access a set of objects. </li> 
 *     <li> <code> OsidNode: </code> An interface used by hierarchy nodes. 
 *     </li> 
 *     <li> <code> OsidCondition: </code> An input or "statement of fact" into 
 *     an <code> OsidRule </code> evaluation. </li> 
 *     <li> <code> OsidInput: </code> An input of source data into an <code> 
 *     OsidRule </code> processor. </li> 
 *     <li> <code> OsidResult: </code> The output from processing an <code> 
 *     OsidRule. </code> </li> 
 *     <li> <code> OsidRecord: </code> An interface marker for an extension to 
 *     another interface. <code> OsidRecord </code> are negotiated using OSID 
 *     <code> Types. </code> </li> 
 * </ul>
 * 
 * 
 * <ul>
 *     <li> <code> Property: </code> Maps a name to a value. Properties are 
 *     available in OSID objects to provide a simplified view of data that may 
 *     exist within a typed interface. </li> 
 *     <li> <code> PropertyList: </code> A list of properties. </li> 
 * </ul>
 * 
 * <h4> Runtime </h4> 
 * 
 * <ul>
 *     <li> <code> OsidRuntimeProfile: </code> The <code> OsidProfile </code> 
 *     for the runtime <code> OsidManager. </code> </li> 
 *     <li> <code> OsidRuntimeManager: </code> The OSID Runtime service. </li> 
 * </ul>
 * 
 * <h4> Abstract Flow </h4> 
 * 
 * <p> Generally, these definitions are abstract and not accesed directly. 
 * They are used as building blocks to define interfaces in the OSIDs 
 * themselves. OSIDs derive most of their definitions from a definition in the 
 * osid package. The methods that are defined at this abstract level versus 
 * the methods defined directly in a specific OSID is determined by the typing 
 * in the method signatures. The osid package interfaces are a means of 
 * ensuring consistency of common methods and not designed to facilitate 
 * object polymorphism among different OSIDs. A language binder may elect to 
 * alter the interface hierarchy presented in this specification and a 
 * provider need not parallel these interfaces in their implementations. </p> 
 * 
 * <p> The flow of control through any OSID can be described in terms of these 
 * definitions. An <code> OsidManager </code> or <code> OsidProxyManager 
 * </code> is retrieved from the <code> OsidRuntimeManager </code> for a given 
 * service. Both types of managers share an interface for describing what they 
 * support in the <code> OsidProfile. </code> </p> 
 * 
 * <p> <code> OsidSessions </code> are created from the <code> OsidManager. 
 * </code> <code> OsidSessions </code> tend to be organized along clusters of 
 * like-functionality. Lookup-oriented sessions retrieve <code> OsidObjects. 
 * </code> Return of multiple <code> OsidObjects </code> is done via the 
 * <code> OsidList. </code> Search-oriented sessions retrieve <code> 
 * OsidObjects </code> through searches provided through the <code> OsidQuery 
 * </code> and <code> OsidSearch </code> interfaces. </p> 
 * 
 * <p> Administrative-oriented sessions create and update <code> OsidObjects 
 * </code> using the <code> OsidForm </code> interface. The <code> OsidForm 
 * </code> makes available <code> Metadata </code> to help define its rules 
 * for setting and changing various data elements. </p> 
 * 
 * <p> <code> OsidObjects </code> can be organized within <code> OsidCatalogs. 
 * </code> An <code> OsidCatalog </code> is hierarchical and can be traversed 
 * through an <code> OsidNode. </code> An <code> OsidQuery </code> or an 
 * <code> OsidSearchOrder </code> may be mapped to a dynamic <code> 
 * OsidCatalog. </code> Such a query may be examined using an <code> 
 * OsidQueryInspector. </code> </p> 
 * 
 * <p> A notification session provides a means for subscribing to events, "a 
 * new object has been created", for example, and these events are received 
 * from an <code> OsidReceiver. </code> </p> 
 * <h4> Meta OSID Specification </h4> 
 * 
 * <p> The OSID Specification framework defines the interace and method 
 * structures as well as the language primitives and errors used throughout 
 * the OSIDs. The OSID Specifications are defined completely in terms of 
 * interfaces and the elements specified in the meta specification. </p> 
 * <h4> Language Primitives </h4> 
 * 
 * <p> Ths meta OSID Specification enumerates the allowable language 
 * primitives that can be used in OSID method signatures. Parameters and 
 * returns in OSID methods may be specified in terms of other OSID interfaces 
 * or using one of these primitives. An OSID Binder translates these language 
 * primitives into an appropriate language primitive counterpart. </p> 
 * 
 * <p> An OSID Primitive differs from a language primitive. An OSID Primitive 
 * is an interface used to describe a more complex structure than a simple 
 * language primitive can support. Both OSID Primitives and language 
 * primitives have the same behavior in the OSIDs in that an there is no 
 * service encapsulation present allowing OSID Primitives to be consructed by 
 * an OSID Consumer. </p> 
 * <h4> Errors </h4> 
 * 
 * <p> OSID methods are required to return a value, if specified, or return 
 * one of the errors specified in the method signature. The meta package 
 * defines the set of errors that a method signtaure may use. </p> 
 * 
 * <p> Errors should result when the contract of the interface as been 
 * violated or cannot be fulfilled and it is necessary to disrupt the flow of 
 * control for a consumer. Different errors are specified where it is forseen 
 * that a consumer may wish to execute a different action without violating 
 * the encapsulation of internal provider operations. Such actions do not 
 * include debugging or other detailed information which is the responsibility 
 * of the provider to manage. As such, the number of errors defined across all 
 * the interfaces is kept to a minimum and the context of the error may vary 
 * from method to method in accordance with the spceification. </p> 
 * 
 * <p> Errors are categorized to convey the audience to which the error 
 * pertains. </p> 
 * 
 * <ul>
 *     <li> User Errors: Errors which may be the result of a user operation 
 *     intended for the user. </li> 
 *     <li> Operational Errors: Errors which may be the result of a system or 
 *     some other problem intended for the user. </li> 
 *     <li> Consumer Contract Errors: Software errors resulting in the use of 
 *     the OSIDs by an OSID Consumer intended for the application programmer. 
 *     These also include integration problems where the OSID Consumer 
 *     bypassed a method to test for support of a service or type. </li> 
 *     <li> Provider Contract Errors: Software errors in the use of an OSID by 
 *     an OSID Provider intended for an implementation programmer. </li> 
 * </ul>
 * 
 * <h4> Compliance </h4> 
 * 
 * <p> OSID methods include a compliance statement indicating whether a method 
 * is required or optional to implement. An optional OSID method is one that 
 * defines an UNIMPLEMENTED error and there is a corresponding method to test 
 * for the existence of an implementation. </p> 
 * <h4> OSID 3K Acknowledgements </h4> 
 * 
 * <ul>
 *     <li> Tom Coppeto (Editor &amp; Architect) </li> 
 *     <li> Scott Thorne (Architect) </li> 
 * </ul>
 * 
 * 
 * <p> The authors gratefully acknowledge the following individuals for their 
 * time, wisdom, and contributions in shaping these specifications. </p> 
 * 
 * <ul>
 *     <li> Adam Franco, Middlebury College </li> 
 *     <li> Jeffrey Merriman, Massachusetts Institute of Technology </li> 
 *     <li> Charles Shubert, Massachusetts Insitute of Technology </li> 
 * </ul>
 * 
 * 
 * <ul>
 *     <li> Prof. Marc Alier, Universitat Polit?cnica de Catalyuna </li> 
 *     <li> Joshua Aresty, Massachusetts Institute of Technology </li> 
 *     <li> Fabrizio Cardinali, Giunti Labs </li> 
 *     <li> Pablo Casado, Universitat Polit?cnica de Catalyuna </li> 
 *     <li> Alex Chapin, Middlebury College </li> 
 *     <li> Craig Counterman, Massachusetts Institute of Technology </li> 
 *     <li> Francesc Santanach Delisau, Universitat Oberta de Catalyuna </li> 
 *     <li> Prof. Lloren? Valverde Garcia, Universitat Oberta de Catalyuna 
 *     </li> 
 *     <li> Catherine Iannuzzo, Massachusetts Institute of Technology </li> 
 *     <li> Jeffrey Kahn, Verbena Consulting </li> 
 *     <li> Michael Korcynski, Tufts University </li> 
 *     <li> Anoop Kumar, Tufts University </li> 
 *     <li> Eva de Lera, Universitat Oberta de Catalyuna </li> 
 *     <li> Roberto Garc?a Marrod?n, Universitat Oberta de Catalyuna </li> 
 *     <li> Andrew McKinney, Massachusetts Institute of Technology </li> 
 *     <li> Scott Morris, Apple </li> 
 *     <li> Mark Norton, Nolaria Consulting </li> 
 *     <li> Mark O'Neill, Dartmouth College </li> 
 *     <li> Prof. Charles Severance, University of Michigan </li> 
 *     <li> Stuart Sim, Sun Microsystems/Common Need </li> 
 *     <li> Colin Smythe, IMS Global Learning Consortium </li> 
 *     <li> George Ward, California State University </li> 
 *     <li> Peter Wilkins, Massachusetts Institute of Technology </li> 
 *     <li> Norman Wright, Massachusetts Institute of Technology </li> 
 * </ul>
 * 
 * <h4> O.K.I. Acknowledgements </h4> 
 * 
 * <p> OSID 3K is based on the O.K.I. OSIDs developed as part of the MIT Open 
 * Knowledge Initiative (O.K.I) project 2001-2004. </p> 
 * 
 * <ul>
 *     <li> Vijay Kumar, O.K.I. Principal Investigator, Massachusetts Insitute 
 *     of Technology </li> 
 *     <li> Jeffrey Merriman, O.K.I. Project Director, Massachusetts Insitute 
 *     of Technology </li> 
 *     <li> Scott Thorne, O.K.I. Chief Architect, Massachusetts Institute of 
 *     Technology </li> 
 *     <li> Charles Shubert, O.K.I. Architect, Massachusetts Institute of 
 *     Technology </li> 
 *     <li> Lois Brooks, Project Coordinator, Stanford University </li> 
 *     <li> Mark Brown, O.K.I. Project Manager, Massachusetts Institute of 
 *     Technology </li> 
 *     <li> Bill Fitzgerald, O.K.I. Finance Manager, Massachusetts Institute 
 *     of Technology </li> 
 *     <li> Judson Harward, Educational Systems Architect, Massachusetts 
 *     Institute of Technology </li> 
 *     <li> Charles Kerns, Educational Systems Architect, Stanford University 
 *     </li> 
 *     <li> Jeffrey Kahn, O.K.I. Partner, Verbena Consulting </li> 
 *     <li> Judith Leonard, O.K.I. Project Administrator, Massachusetts 
 *     Institute of Technology </li> 
 *     <li> Phil Long, O.K.I. Outreach Coordinator, Massachusetts Institute of 
 *     Technology </li> 
 * </ul>
 * 
 * 
 * <ul>
 *     <li> Cambridge University, O.K.I. Core Collaborator </li> 
 *     <li> Dartmouth College, O.K.I. Core Collaborator </li> 
 *     <li> Massachusetts Institute of Technology, O.K.I. Core Collaborator 
 *     </li> 
 *     <li> North Carolina State University, O.K.I. Core Collaborator </li> 
 *     <li> Stanford University, O.K.I. Core Collaborator </li> 
 *     <li> University of Michigan, O.K.I. Core Collaborator </li> 
 *     <li> University of Pennsylvania, O.K.I. Core Collaborator </li> 
 *     <li> University of Wisconsin, Madison, O.K.I. Core Collaborator </li> 
 * </ul>
 * 
 * 
 * <br/><hr/><br/>
 * 
 * <p>Copyright &copy; 2002-2004, 2006-2008 Massachusetts Institute of 
 * Technology. </p> 
 * 
 * <p> Copyright &copy; 2009-2012 Ingenescus. All Rights Reserved. </p>
 * 
 * <p>This Work is being provided by the copyright holder(s) subject to the 
 * following license. By obtaining, using and/or copying this Work, you agree 
 * that you have read, understand, and will comply with the following terms 
 * and conditions. </p> 
 * 
 * <p> Permission to use, copy and distribute unmodified versions of this 
 * Work, for any purpose, without fee or royalty is hereby granted, provided 
 * that you include the above copyright notices and the terms of this license 
 * on ALL copies of the Work or portions thereof. </p> 
 * 
 * <p> You may modify or create Derivatives of this Work only for your 
 * internal purposes. You shall not distribute or transfer any such Derivative 
 * of this Work to any location or to any third party. For the purposes of 
 * this license, "Derivative" shall mean any derivative of the Work as defined 
 * in the United States Copyright Act of 1976, such as a translation or 
 * modification. </p> 
 * 
 * <p> This Work and the information contained herein is provided on an "AS 
 * IS" basis WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT 
 * NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
 * PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT 
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN 
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN 
 * CONNECTION WITH THE WORK OR THE USE OR OTHER DEALINGS IN THE WORK. </p> 
 * 
 * <p> The export of software employing encryption technology may require a 
 * specific license from the United States Government. It is the 
 * responsibility of any person or organization contemplating export to obtain 
 * such a license before exporting this Work. </p>
 */

package org.osid;
