//
// org.osid.resourcing.AvailabilityLookupSession
//
//     The OSID Java specification for org.osid.resourcing.AvailabilityLookupSession.
//
// Copyright (C) 2010 Ingenescus. All Rights Reserved. 
//
//     This Work is being provided by the copyright holder(s) subject to the 
//     following license. By obtaining, using and/or copying this Work, you 
//     agree that you have read, understand, and will comply with the 
//     following terms and conditions. 
//     
//     Permission to use, copy and distribute unmodified versions of this 
//     Work, for any purpose, without fee or royalty is hereby granted, 
//     provided that you include the above copyright notices and the terms of 
//     this license on ALL copies of the Work or portions thereof. 
//     
//     You may modify or create Derivatives of this Work only for your 
//     internal purposes. You shall not distribute or transfer any such 
//     Derivative of this Work to any location or to any third party. For the 
//     purposes of this license, "Derivative" shall mean any derivative of the 
//     Work as defined in the United States Copyright Act of 1976, such as a 
//     translation or modification. 
//     
//     This Work and the information contained herein is provided on an "AS 
//     IS" basis WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING 
//     BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
//     PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS 
//     OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
//     LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
//     FROM, OUT OF OR IN CONNECTION WITH THE WORK OR THE USE OR OTHER 
//     DEALINGS IN THE WORK. 
//     
//     The export of software employing encryption technology may require a 
//     specific license from the United States Government. It is the 
//     responsibility of any person or organization contemplating export to 
//     obtain such a license before exporting this Work. 
//

package org.osid.resourcing;


/**
 *  <p>This session defines methods for retrieving availabilities. </p> 
 *  
 *  <p> This lookup session defines several views: </p> 
 *  
 *  <ul>
 *      <li> comparative view: elements may be silently omitted or re-ordered 
 *      </li> 
 *      <li> plenary view: provides a complete result set or is an error 
 *      condition </li> 
 *      <li> isolated foundry view: All availability methods in this session 
 *      operate, retrieve and pertain to availabilities defined explicitly in 
 *      the current foundry. Using an isolated view is useful for managing 
 *      <code> Availabilities </code> with the <code> 
 *      AvailabilityAdminSession. </code> </li> 
 *      <li> federated foundry view: All availability lookup methods in this 
 *      session operate, retrieve and pertain to all availabilities defined in 
 *      this foundry and any other availabilities implicitly available in this 
 *      foundry through foundry inheritence. </li> 
 *      <li> effective availability view: All availability lookup methods 
 *      return availabilities where the current date falls in between the 
 *      effective dates inclusive. </li> 
 *      <li> any effective availability view: Availabilities of any effective 
 *      date are returned from methods. </li> 
 *  </ul>
 *  
 *  
 *  <p> The methods <code> useFederatedFoundryView() </code> and <code> 
 *  useIsolatedFoundryView() </code> behave as a radio group and one should be 
 *  selected before invoking any lookup methods. </p> 
 *  
 *  <p> Availabilities may have an additional records indicated by their 
 *  respective record types. The record may not be accessed through a cast of 
 *  the <code> Availabilities. </code> </p>
 */

public interface AvailabilityLookupSession
    extends org.osid.OsidSession {


    /**
     *  Gets the <code> Foundry </code> <code> Id </code> associated with this 
     *  session. 
     *
     *  @return the <code> Foundry Id </code> associated with this session 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.id.Id getFoundryId();


    /**
     *  Gets the <code> Foundry </code> associated with this session. 
     *
     *  @return the foundry 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.resourcing.Foundry getFoundry()
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Tests if this user can perform <code> Availability </code> lookups. A 
     *  return of true does not guarantee successful authorization. A return 
     *  of false indicates that it is known all methods in this session will 
     *  result in a <code> PERMISSION_DENIED. </code> This is intended as a 
     *  hint to an application that may not offer lookup operations to 
     *  unauthorized users. 
     *
     *  @return <code> false </code> if lookup methods are not authorized, 
     *          <code> true </code> otherwise 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean canLookupAvailabilities();


    /**
     *  The returns from the lookup methods may omit or translate elements 
     *  based on this session, such as authorization, and not result in an 
     *  error. This view is used when greater interoperability is desired at 
     *  the expense of precision. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useComparativeAvailabilityView();


    /**
     *  A complete view of the <code> Availability </code> returns is desired. 
     *  Methods will return what is requested or result in an error. This view 
     *  is used when greater precision is desired at the expense of 
     *  interoperability. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void usePlenaryAvailabilityView();


    /**
     *  Federates the view for methods in this session. A federated view will 
     *  include availabilities in foundries which are children of this foundry 
     *  in the foundry hierarchy. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useFederatedFoundryView();


    /**
     *  Isolates the view for methods in this session. An isolated view 
     *  restricts retrievals to this foundry only. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useIsolatedFoundryView();


    /**
     *  Only availabilities whose effective dates are current are returned by 
     *  methods in this session. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useEffectiveAvailabilityView();


    /**
     *  All availabilities of any effective dates are returned by methods in 
     *  this session. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useAnyEffectiveAvailabilityView();


    /**
     *  Gets the <code> Availability </code> specified by its <code> Id. 
     *  </code> 
     *  
     *  <code> </code> In plenary mode, the exact <code> Id </code> is found 
     *  or a <code> NOT_FOUND </code> results. Otherwise, the returned <code> 
     *  Availability </code> may have a different <code> Id </code> than 
     *  requested, such as the case where a duplicate <code> Id </code> was 
     *  assigned to an <code> Availability </code> and retained for 
     *  compatibility. 
     *  
     *  In effective mode, availabilities are returned that are currently 
     *  effective. In any effective mode, effective availabilities and those 
     *  currently expired are returned. 
     *
     *  @param  availabilityId the <code> Id </code> of the <code> 
     *          Availability </code> to retrieve 
     *  @return the returned <code> Availability </code> 
     *  @throws org.osid.NotFoundException no <code> Availability </code> 
     *          found with the given <code> Id </code> 
     *  @throws org.osid.NullArgumentException <code> availabilityId </code> 
     *          is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.resourcing.Availability getAvailability(org.osid.id.Id availabilityId)
        throws org.osid.NotFoundException,
               org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> AvailabilityList </code> corresponding to the given 
     *  <code> IdList. </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all of the 
     *  availabilities specified in the <code> Id </code> list, in the order 
     *  of the list, including duplicates, or an error results if an <code> Id 
     *  </code> in the supplied list is not found or inaccessible. Otherwise, 
     *  inaccessible <code> Availabilities </code> may be omitted from the 
     *  list and may present the elements in any order including returning a 
     *  unique set. 
     *  
     *  In effective mode, availabilities are returned that are currently 
     *  effective. In any effective mode, effective availabilities and those 
     *  currently expired are returned. 
     *
     *  @param  availabilityIds the list of <code> Ids </code> to retrieve 
     *  @return the returned <code> Availability </code> list 
     *  @throws org.osid.NotFoundException an <code> Id </code> was not found 
     *  @throws org.osid.NullArgumentException <code> availabilityIds </code> 
     *          is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.resourcing.AvailabilityList getAvailabilitiesByIds(org.osid.id.IdList availabilityIds)
        throws org.osid.NotFoundException,
               org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> AvailabilityList </code> corresponding to the given 
     *  availability genus <code> Type </code> which does not include 
     *  availabilities of types derived from the specified <code> Type. 
     *  </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  availabilities or an error results. Otherwise, the returned list may 
     *  contain only those availabilities that are accessible through this 
     *  session. 
     *  
     *  In effective mode, availabilities are returned that are currently 
     *  effective. In any effective mode, effective availabilities and those 
     *  currently expired are returned. 
     *
     *  @param  availabilityGenusType an availability genus type 
     *  @return the returned <code> Availability </code> list 
     *  @throws org.osid.NullArgumentException <code> availabilityGenusType 
     *          </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.resourcing.AvailabilityList getAvailabilitiesByGenusType(org.osid.type.Type availabilityGenusType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> AvailabilityList </code> corresponding to the given 
     *  availability genus <code> Type </code> and include any additional 
     *  availabilities with genus types derived from the specified <code> 
     *  Type. </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  availabilities or an error results. Otherwise, the returned list may 
     *  contain only those availabilities that are accessible through this 
     *  session. 
     *  
     *  In effective mode, availabilities are returned that are currently 
     *  effective. In any effective mode, effective availabilities and those 
     *  currently expired are returned. 
     *
     *  @param  availabilityGenusType an availability genus type 
     *  @return the returned <code> Availability </code> list 
     *  @throws org.osid.NullArgumentException <code> availabilityGenusType 
     *          </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.resourcing.AvailabilityList getAvailabilitiesByParentGenusType(org.osid.type.Type availabilityGenusType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> AvailabilityList </code> containing the given 
     *  availability record <code> Type. </code> 
     *  
     *  In plenary mode, the returned list contains all known availabilities 
     *  or an error results. Otherwise, the returned list may contain only 
     *  those availabilities that are accessible through this session. 
     *  
     *  In effective mode, availabilities are returned that are currently 
     *  effective. In any effective mode, effective availabilities and those 
     *  currently expired are returned. 
     *
     *  @param  availabilityRecordType an availability record type 
     *  @return the returned <code> Availability </code> list 
     *  @throws org.osid.NullArgumentException <code> availabilityRecordType 
     *          </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.resourcing.AvailabilityList getAvailabilitiesByRecordType(org.osid.type.Type availabilityRecordType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of availabilities effective during the entire given date 
     *  range inclusive but not confined to the date range. <code> </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  availabilities or an error results. Otherwise, the returned list may 
     *  contain only those availabilities that are accessible through this 
     *  session. 
     *  
     *  In effective mode, availabilities are returned that are currently 
     *  effective. In any effective mode, effective availabilities and those 
     *  currently expired are returned. 
     *
     *  @param  from start of date range 
     *  @param  to end of date range 
     *  @return the returned <code> Availability </code> list 
     *  @throws org.osid.InvalidArgumentException <code> from </code> is 
     *          greater than <code> to </code> 
     *  @throws org.osid.NullArgumentException <code> from </code> or <code> 
     *          to </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.resourcing.AvailabilityList getAvailabilitiesOnDate(org.osid.calendaring.DateTime from, 
                                                                        org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of availabilities for a resource. 
     *  
     *  In plenary mode, the returned list contains all known availabilities 
     *  or an error results. Otherwise, the returned list may contain only 
     *  those availabilities that are accessible through this session. 
     *  
     *  In effective mode, availabilities are returned that are currently 
     *  effective. In any effective mode, effective availabilities and those 
     *  currently expired are returned. 
     *
     *  @param  resourceId a resource <code> Id </code> 
     *  @return the returned <code> Availability </code> list 
     *  @throws org.osid.NullArgumentException <code> resourceId </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.resourcing.AvailabilityList getAvailabilitiesForResource(org.osid.id.Id resourceId)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of availabilities for a resource and effective during the 
     *  entire given date range inclusive but not confined to the date range. 
     *  <code> </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  availabilities or an error results. Otherwise, the returned list may 
     *  contain only those availabilities that are accessible through this 
     *  session. 
     *  
     *  In effective mode, availabilities are returned that are currently 
     *  effective. In any effective mode, effective availabilities and those 
     *  currently expired are returned. 
     *
     *  @param  resourceId a resource <code> Id </code> 
     *  @param  from start of date range 
     *  @param  to end of date range 
     *  @return the returned <code> Availability </code> list 
     *  @throws org.osid.InvalidArgumentException <code> from </code> is 
     *          greater than <code> to </code> 
     *  @throws org.osid.NullArgumentException <code> resourceId, from, 
     *          </code> or <code> to </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.resourcing.AvailabilityList getAvailabilitiesForResourceOnDate(org.osid.id.Id resourceId, 
                                                                                   org.osid.calendaring.DateTime from, 
                                                                                   org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of availabilities for a job. <code> </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  availabilities or an error results. Otherwise, the returned list may 
     *  contain only those availabilities that are accessible through this 
     *  session. 
     *  
     *  In effective mode, availabilities are returned that are currently 
     *  effective. In any effective mode, effective availabilities and those 
     *  currently expired are returned. 
     *
     *  @param  jobId a job <code> Id </code> 
     *  @return the returned <code> Availability </code> list 
     *  @throws org.osid.NullArgumentException <code> jobId </code> is <code> 
     *          null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.resourcing.AvailabilityList getAvailabilitiesForJob(org.osid.id.Id jobId)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of availabilities for a job and effective during the 
     *  entire given date range inclusive but not confined to the date range. 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  availabilities or an error results. Otherwise, the returned list may 
     *  contain only those availabilities that are accessible through this 
     *  session. 
     *  
     *  In effective mode, availabilities are returned that are currently 
     *  effective. In any effective mode, effective availabilities and those 
     *  currently expired are returned. 
     *
     *  @param  jobId a job <code> Id </code> 
     *  @param  from start of date range 
     *  @param  to end of date range 
     *  @return the returned <code> Availability </code> list 
     *  @throws org.osid.InvalidArgumentException <code> from </code> is 
     *          greater than <code> to </code> 
     *  @throws org.osid.NullArgumentException <code> jobId, from, </code> or 
     *          <code> to </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.resourcing.AvailabilityList getAvailabilitiesForJobOnDate(org.osid.id.Id jobId, 
                                                                              org.osid.calendaring.DateTime from, 
                                                                              org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of availabilities for a resource and job. <code> </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  availabilities or an error results. Otherwise, the returned list may 
     *  contain only those availabilities that are accessible through this 
     *  session. 
     *  
     *  In effective mode, credits are returned that are currently effective. 
     *  In any effective mode, effective credits and those currently expired 
     *  are returned. 
     *
     *  @param  resourceId a resource <code> Id </code> 
     *  @param  jobId a job <code> Id </code> 
     *  @return the returned <code> Availability </code> list 
     *  @throws org.osid.NullArgumentException <code> resourceId </code> or 
     *          <code> jobId </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.resourcing.AvailabilityList getAvailabilitiesForResourceAndJob(org.osid.id.Id resourceId, 
                                                                                   org.osid.id.Id jobId)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets a list of availabilities for a resource and job and effective 
     *  during the entire given date range inclusive but not confined to the 
     *  date range. <code> </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  availabilities or an error results. Otherwise, the returned list may 
     *  contain only those availabilities that are accessible through this 
     *  session. 
     *  
     *  In effective mode, availabilities are returned that are currently 
     *  effective. In any effective mode, effective availabilities and those 
     *  currently expired are returned. 
     *
     *  @param  resourceId a resource <code> Id </code> 
     *  @param  jobId a job <code> Id </code> 
     *  @param  from start of date range 
     *  @param  to end of date range 
     *  @return the returned <code> Availability </code> list 
     *  @throws org.osid.InvalidArgumentException <code> from </code> is 
     *          greater than <code> to </code> 
     *  @throws org.osid.NullArgumentException <code> resourceId, jobId, from, 
     *          </code> or <code> to </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.resourcing.AvailabilityList getAvailabilitiesForResourceAndJobOnDate(org.osid.id.Id resourceId, 
                                                                                         org.osid.id.Id jobId, 
                                                                                         org.osid.calendaring.DateTime from, 
                                                                                         org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets all availabilities. <code> </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all of the 
     *  availabilities, including duplicates, or an error results if an <code> 
     *  Id </code> in the supplied list is not found or inaccessible. 
     *  Otherwise, inaccessible <code> Availabilities </code> may be omitted 
     *  from the list and may present the elements in any order including 
     *  returning a unique set. 
     *  
     *  In effective mode, availabilities are returned that are currently 
     *  effective. In any effective mode, effective availabilities and those 
     *  currently expired are returned. 
     *
     *  @return the returned <code> Availability </code> list 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.resourcing.AvailabilityList getAvailabilities()
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;
}
