//
// org.osid.topology.EdgeLookupSession
//
//     The OSID Java specification for org.osid.topology.EdgeLookupSession.
//
// Copyright (C) 2010 Ingenescus. All Rights Reserved. 
//
//     This Work is being provided by the copyright holder(s) subject to the 
//     following license. By obtaining, using and/or copying this Work, you 
//     agree that you have read, understand, and will comply with the 
//     following terms and conditions. 
//     
//     Permission to use, copy and distribute unmodified versions of this 
//     Work, for any purpose, without fee or royalty is hereby granted, 
//     provided that you include the above copyright notices and the terms of 
//     this license on ALL copies of the Work or portions thereof. 
//     
//     You may modify or create Derivatives of this Work only for your 
//     internal purposes. You shall not distribute or transfer any such 
//     Derivative of this Work to any location or to any third party. For the 
//     purposes of this license, "Derivative" shall mean any derivative of the 
//     Work as defined in the United States Copyright Act of 1976, such as a 
//     translation or modification. 
//     
//     This Work and the information contained herein is provided on an "AS 
//     IS" basis WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING 
//     BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A 
//     PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS 
//     OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
//     LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
//     FROM, OUT OF OR IN CONNECTION WITH THE WORK OR THE USE OR OTHER 
//     DEALINGS IN THE WORK. 
//     
//     The export of software employing encryption technology may require a 
//     specific license from the United States Government. It is the 
//     responsibility of any person or organization contemplating export to 
//     obtain such a license before exporting this Work. 
//

package org.osid.topology;


/**
 *  <p>This session provides methods for retrieving <code> Edge </code> 
 *  objects. The <code> Edge </code> represents a connection between two 
 *  <code> Nodes. </code> </p> 
 *  
 *  <p> This session defines views that offer differing behaviors when 
 *  retrieving multiple objects. </p> 
 *  
 *  <ul>
 *      <li> comparative view: elements may be silently omitted or re-ordered 
 *      </li> 
 *      <li> plenary view: provides a complete set or is an error condition 
 *      </li> 
 *      <li> isolated graph view: All edge methods in this session operate, 
 *      retrieve and pertain to edges defined explicitly in the current graph. 
 *      Using an isolated view is useful for managing <code> Edges </code> 
 *      with the <code> EdgeAdminSession. </code> </li> 
 *      <li> federated graph view: All edge methods in this session operate, 
 *      retrieve and pertain to all edges defined in this graph and any other 
 *      edges implicitly available in this graph through graph inheritence. 
 *      </li> 
 *      <li> effective edge view: All edge lookup methods return edges where 
 *      the current dates falls in between the effective dates inclusive. 
 *      </li> 
 *      <li> any effective edge view: Edges of any effective date are 
 *      returned. </li> 
 *  </ul>
 *  
 *  
 *  <p> Generally, the comparative view should be used for most applications 
 *  as it permits operation even if there is data that cannot be accessed. The 
 *  methods <code> useFederatedGraphView() </code> and <code> 
 *  useIsolatedGraphView() </code> behave as a radio group and one should be 
 *  selected before invoking any lookup methods. </p>
 */

public interface EdgeLookupSession
    extends org.osid.OsidSession {


    /**
     *  Gets the <code> Graph </code> <code> Id </code> associated with this 
     *  session. 
     *
     *  @return the <code> Graph Id </code> associated with this session 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.id.Id getGraphId();


    /**
     *  Gets the <code> Graph </code> associated with this session. 
     *
     *  @return the graph 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.topology.Graph getGraph()
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Tests if this user can perform <code> Edge </code> lookups. A return 
     *  of true does not guarantee successful authorization. A return of false 
     *  indicates that it is known all methods in this session will result in 
     *  a <code> PERMISSION_DENIED. </code> This is intended as a hint to an 
     *  application that may opt not to offer lookup operations to 
     *  unauthorized users. 
     *
     *  @return <code> false </code> if lookup methods are not authorized, 
     *          <code> true </code> otherwise 
     *  @compliance mandatory This method must be implemented. 
     */

    public boolean canLookupEdges();


    /**
     *  The returns from the lookup methods may omit or translate elements 
     *  based on this session, such as authorization, and not result in an 
     *  error. This view is used when greater interoperability is desired at 
     *  the expense of precision. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useComparativeEdgeView();


    /**
     *  A complete view of the <code> Edge </code> returns is desired. Methods 
     *  will return what is requested or result in an error. This view is used 
     *  when greater precision is desired at the expense of interoperability. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void usePlenaryEdgeView();


    /**
     *  Federates the view for methods in this session. A federated view will 
     *  include edges in graphs which are children of this graph in the graph 
     *  hierarchy. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useFederatedGraphView();


    /**
     *  Isolates the view for methods in this session. An isolated view 
     *  restricts lookups to this graph only. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useIsolatedGraphView();


    /**
     *  Only edges whose effective dates are current are returned by methods 
     *  in this session. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useEffectiveEdgeView();


    /**
     *  All edges of any effective dates are returned by all methods in this 
     *  session. 
     *
     *  @compliance mandatory This method is must be implemented. 
     */

    public void useAnyEffectiveEdgeView();


    /**
     *  Gets the <code> Edge </code> specified by its <code> Id. </code> 
     *  
     *  <code> </code> In plenary mode, the exact <code> Id </code> is found 
     *  or a <code> NOT_FOUND </code> results. Otherwise, the returned <code> 
     *  Edge </code> may have a different <code> Id </code> than requested, 
     *  such as the case where a duplicate <code> Id </code> was assigned to 
     *  an <code> Edge </code> and retained for compatibility. 
     *  
     *  In effective mode, edges are returned that are currently effective. In 
     *  any effective mode, effective edges and those currently expired are 
     *  returned. 
     *
     *  @param  edgeId <code> Id </code> of the <code> Edge </code> 
     *  @return the edge 
     *  @throws org.osid.NotFoundException <code> edgeId </code> not found 
     *  @throws org.osid.NullArgumentException <code> edgeId </code> is <code> 
     *          null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method is must be implemented. 
     */

    public org.osid.topology.Edge getEdge(org.osid.id.Id edgeId)
        throws org.osid.NotFoundException,
               org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> EdgeList </code> corresponding to the given <code> 
     *  IdList. </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all of the 
     *  edges specified in the <code> Id </code> list, in the order of the 
     *  list, including duplicates, or an error results if an <code> Id 
     *  </code> in the supplied list is not found or inaccessible. Otherwise, 
     *  inaccessible <code> Edges </code> may be omitted from the list and may 
     *  present the elements in any order including returning a unique set. 
     *  
     *  In effective mode, edges are returned that are currently effective. In 
     *  any effective mode, effective edges and those currently expired are 
     *  returned. 
     *
     *  @param  edgeIds the list of <code> Ids </code> to retrieve 
     *  @return the returned <code> Edge </code> list 
     *  @throws org.osid.NotFoundException an <code> Id was </code> not found 
     *  @throws org.osid.NullArgumentException <code> edgeIds </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.topology.EdgeList getEdgesByIds(org.osid.id.IdList edgeIds)
        throws org.osid.NotFoundException,
               org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> EdgeList </code> corresponding to the given edge genus 
     *  <code> Type </code> which does not include edges of genus types 
     *  derived from the specified <code> Type. </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  edges or an error results. Otherwise, the returned list may contain 
     *  only those edges that are accessible through this session. 
     *  
     *  In effective mode, edges are returned that are currently effective. In 
     *  any effective mode, effective edges and those currently expired are 
     *  returned. 
     *
     *  @param  edgeGenusType an edge genus type 
     *  @return the returned <code> Edge </code> list 
     *  @throws org.osid.NullArgumentException <code> edgeGenusType </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.topology.EdgeList getEdgesByGenusType(org.osid.type.Type edgeGenusType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> EdgeList </code> corresponding to the given edge genus 
     *  <code> Type </code> and include any additional edges with genus types 
     *  derived from the specified <code> Type. </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  edges or an error results. Otherwise, the returned list may contain 
     *  only those edges that are accessible through this session. 
     *  
     *  In effective mode, edges are returned that are currently effective. In 
     *  any effective mode, effective edges and those currently expired are 
     *  returned. 
     *
     *  @param  edgeGenusType an edge genus type 
     *  @return the returned <code> Edge </code> list 
     *  @throws org.osid.NullArgumentException <code> edgeGenusType </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.topology.EdgeList getEdgesByParentGenusType(org.osid.type.Type edgeGenusType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> EdgeList </code> containing the given edge record 
     *  <code> Type. </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  edges or an error results. Otherwise, the returned list may contain 
     *  only those edges that are accessible through this session. 
     *  
     *  In effective mode, edges are returned that are currently effective. In 
     *  any effective mode, effective edges and those currently expired are 
     *  returned. 
     *
     *  @param  edgeRecordType an edge record type 
     *  @return the returned <code> Edge </code> list 
     *  @throws org.osid.NullArgumentException <code> edgeRecordType </code> 
     *          is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.topology.EdgeList getEdgesByRecordType(org.osid.type.Type edgeRecordType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> EdgeList </code> effective during the entire given date 
     *  range inclusive but not confined to the date range. <code> </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  edges or an error results. Otherwise, the returned list may contain 
     *  only those edges that are accessible through this session. 
     *  
     *  In effective mode, edges are returned that are currently effective. In 
     *  any effective mode, effective edges and those currently expired are 
     *  returned. 
     *
     *  @param  from starting date 
     *  @param  to ending date 
     *  @return the returned <code> Edge </code> list 
     *  @throws org.osid.InvalidArgumentException <code> from </code> is 
     *          greater than <code> to </code> 
     *  @throws org.osid.NullArgumentException <code> from </code> or <code> 
     *          to </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.topology.EdgeList getEdgesOnDate(org.osid.calendaring.DateTime from, 
                                                     org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> EdgeList </code> of a genus type effective during the 
     *  entire given date range inclusive but not confined to the date range. 
     *  <code> </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  edges or an error results. Otherwise, the returned list may contain 
     *  only those edges that are accessible through this session. 
     *  
     *  In effective mode, edges are returned that are currently effective. In 
     *  any effective mode, effective edges and those currently expired are 
     *  returned. 
     *
     *  @param  edgeGenusType an edge genus type 
     *  @param  from starting date 
     *  @param  to ending date 
     *  @return the returned <code> Edge </code> list 
     *  @throws org.osid.InvalidArgumentException <code> from </code> is 
     *          greater than <code> to </code> 
     *  @throws org.osid.NullArgumentException <code> edgeGenusType, from, 
     *          </code> or <code> to </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.topology.EdgeList getEdgesByGenusTypeOnDate(org.osid.type.Type edgeGenusType, 
                                                                org.osid.calendaring.DateTime from, 
                                                                org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> EdgeList </code> corresponding to the given source node 
     *  <code> Id. </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all of the 
     *  edges corresponding to the given peer, including duplicates, or an 
     *  error results if an edge is inaccessible. Otherwise, inaccessible 
     *  <code> Edges </code> may be omitted from the list and may present the 
     *  elements in any order including returning a unique set. 
     *  
     *  In effective mode, edges are returned that are currently effective. In 
     *  any effective mode, effective edges and those currently expired are 
     *  returned. 
     *
     *  @param  sourceNodeId a node <code> Id </code> 
     *  @return a list of edges 
     *  @throws org.osid.NullArgumentException <code> sourceNodeId </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.topology.EdgeList getEdgesForSourceNode(org.osid.id.Id sourceNodeId)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> EdgeList </code> with effective during the entire given 
     *  date range inclusive but not confined to the date range. <code> 
     *  </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  edges or an error results. Otherwise, the returned list may contain 
     *  only those edges that are accessible through this session. 
     *  
     *  In effective mode, edges are returned that are currently effective. In 
     *  any effective mode, effective edges and those currently expired are 
     *  returned. 
     *
     *  @param  sourceNodeId a node <code> Id </code> 
     *  @param  from starting date 
     *  @param  to ending date 
     *  @return a list of edges 
     *  @throws org.osid.InvalidArgumentException <code> from </code> is 
     *          greater than <code> to </code> 
     *  @throws org.osid.NullArgumentException <code> sourceNodeId, from 
     *          </code> or <code> to </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.topology.EdgeList getEdgesForSourceNodeOnDate(org.osid.id.Id sourceNodeId, 
                                                                  org.osid.calendaring.DateTime from, 
                                                                  org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> EdgeList </code> corresponding to the given source node 
     *  <code> Id </code> and edge genus <code> Type </code> and includes any 
     *  genus types derived from the given genus type. 
     *  
     *  In plenary mode, the returned list contains all of the edges 
     *  corresponding to the given peer, including duplicates, or an error 
     *  results if an edge is inaccessible. Otherwise, inaccessible <code> 
     *  Edges </code> may be omitted from the list and may present the 
     *  elements in any order including returning a unique set. 
     *  
     *  In effective mode, edges are returned that are currently effective. In 
     *  any effective mode, effective edges and those currently expired are 
     *  returned. 
     *
     *  @param  sourceNodeId a node <code> Id </code> 
     *  @param  edgeGenusType an edge genus type 
     *  @return a list of edges 
     *  @throws org.osid.NullArgumentException <code> sourceNodeId </code> or 
     *          <code> edgeGenusType </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.topology.EdgeList getEdgesByGenusTypeForSourceNode(org.osid.id.Id sourceNodeId, 
                                                                       org.osid.type.Type edgeGenusType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> EdgeList </code> with the given genus type and 
     *  effective during the entire given date range inclusive but not 
     *  confined to the date range. <code> </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  edges or an error results. Otherwise, the returned list may contain 
     *  only those edges that are accessible through this session. 
     *  
     *  In effective mode, edges are returned that are currently effective. In 
     *  any effective mode, effective edges and those currently expired are 
     *  returned. 
     *
     *  @param  nodeId a node <code> Id </code> 
     *  @param  edgeGenusType an edge genus type 
     *  @param  from starting date 
     *  @param  to ending date 
     *  @return a list of edges 
     *  @throws org.osid.InvalidArgumentException <code> from </code> is 
     *          greater than <code> to </code> 
     *  @throws org.osid.NullArgumentException <code> sourceNodeId, 
     *          edgeGenusType, from </code> or <code> to </code> is <code> 
     *          null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.topology.EdgeList getEdgesByGenusTypeForSourceNodeOnDate(org.osid.id.Id nodeId, 
                                                                             org.osid.type.Type edgeGenusType, 
                                                                             org.osid.calendaring.DateTime from, 
                                                                             org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> EdgeList </code> corresponding to the given destination 
     *  node <code> Id. </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all of the 
     *  edges corresponding to the given peer, including duplicates, or an 
     *  error results if an edge is inaccessible. Otherwise, inaccessible 
     *  <code> Edges </code> may be omitted from the list and may present the 
     *  elements in any order including returning a unique set. 
     *  
     *  In effective mode, edges are returned that are currently effective. In 
     *  any effective mode, effective edges and those currently expired are 
     *  returned. 
     *
     *  @param  destinationNodeId a node <code> Id </code> 
     *  @return a list of edges 
     *  @throws org.osid.NullArgumentException <code> destinationNodeId 
     *          </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.topology.EdgeList getEdgesForDestinationNode(org.osid.id.Id destinationNodeId)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> EdgeList </code> effective during the entire given date 
     *  range inclusive but not confined to the date range. <code> </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  edges or an error results. Otherwise, the returned list may contain 
     *  only those edges that are accessible through this session. 
     *  
     *  In effective mode, edges are returned that are currently effective. In 
     *  any effective mode, effective edges and those currently expired are 
     *  returned. 
     *
     *  @param  destinationNodeId a node <code> Id </code> 
     *  @param  from starting date 
     *  @param  to ending date 
     *  @return a list of edges 
     *  @throws org.osid.InvalidArgumentException <code> from </code> is 
     *          greater than <code> to </code> 
     *  @throws org.osid.NullArgumentException <code> destinationNodeId, from 
     *          </code> or <code> to </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.topology.EdgeList getEdgesForDestinationNodeOnDate(org.osid.id.Id destinationNodeId, 
                                                                       org.osid.calendaring.DateTime from, 
                                                                       org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> EdgeList </code> corresponding to the given destination 
     *  node <code> Id </code> and edge genus <code> Type </code> and includes 
     *  any genus types derived from the given genus type. 
     *  
     *  In plenary mode, the returned list contains all of the edges 
     *  corresponding to the given peer, including duplicates, or an error 
     *  results if an edge is inaccessible. Otherwise, inaccessible <code> 
     *  Edges </code> may be omitted from the list and may present the 
     *  elements in any order including returning a unique set. 
     *  
     *  In effective mode, edges are returned that are currently effective. In 
     *  any effective mode, effective edges and those currently expired are 
     *  returned. 
     *
     *  @param  destinationNodeId a node <code> Id </code> 
     *  @param  edgeGenusType an edge genus type 
     *  @return a list of edges 
     *  @throws org.osid.NullArgumentException <code> destinationNodeId 
     *          </code> or <code> edgeGenusType </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.topology.EdgeList getEdgesByGenusTypeForDestinationNode(org.osid.id.Id destinationNodeId, 
                                                                            org.osid.type.Type edgeGenusType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> EdgeList </code> with the given genus type and 
     *  effective during the entire given date range inclusive but not 
     *  confined to the date range. <code> </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  edges or an error results. Otherwise, the returned list may contain 
     *  only those edges that are accessible through this session. 
     *  
     *  In effective mode, edges are returned that are currently effective. In 
     *  any effective mode, effective edges and those currently expired are 
     *  returned. 
     *
     *  @param  destinationNodeId a node <code> Id </code> 
     *  @param  edgeGenusType an edge genus type 
     *  @param  from starting date 
     *  @param  to ending date 
     *  @return a list of edges 
     *  @throws org.osid.InvalidArgumentException <code> from </code> is 
     *          greater than <code> to </code> 
     *  @throws org.osid.NullArgumentException <code> destinationNodeId, 
     *          edgeGenusType, from </code> or <code> to </code> is <code> 
     *          null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.topology.EdgeList getEdgesByGenusTypeForDestinationNodeOnDate(org.osid.id.Id destinationNodeId, 
                                                                                  org.osid.type.Type edgeGenusType, 
                                                                                  org.osid.calendaring.DateTime from, 
                                                                                  org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an EdgeList corresponding to the given node <code> Ids. </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all of the 
     *  edges corresponding to the given peer, including duplicates, or an 
     *  error results if an edge is inaccessible. Otherwise, inaccessible 
     *  <code> Edges </code> may be omitted from the list and may present the 
     *  elements in any order including returning a unique set. 
     *  
     *  In effective mode, edges are returned that are currently effective. In 
     *  any effective mode, effective edges and those currently expired are 
     *  returned. 
     *
     *  @param  sourceNodeId a node <code> Id </code> 
     *  @param  destinationNodeId a node <code> Id </code> 
     *  @return a list of edges 
     *  @throws org.osid.NullArgumentException <code> sourceNodeId </code> or 
     *          <code> destinationNodeId </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.topology.EdgeList getEdgesForNodes(org.osid.id.Id sourceNodeId, 
                                                       org.osid.id.Id destinationNodeId)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> EdgeList </code> effective during the entire given date 
     *  range inclusive but not confined to the date range. <code> </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  edges or an error results. Otherwise, the returned list may contain 
     *  only those edges that are accessible through this session. 
     *  
     *  In effective mode, edges are returned that are currently effective. In 
     *  any effective mode, effective edges and those currently expired are 
     *  returned. 
     *
     *  @param  sourceNodeId a node <code> Id </code> 
     *  @param  destinationNodeId a node <code> Id </code> 
     *  @param  from starting date 
     *  @param  to ending date 
     *  @return a list of edges 
     *  @throws org.osid.InvalidArgumentException <code> from </code> is 
     *          greater than <code> to </code> 
     *  @throws org.osid.NullArgumentException <code> sourceNodeId, 
     *          destinationNodeId, from </code> or <code> to </code> is <code> 
     *          null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.topology.EdgeList getEdgesForNodesOnDate(org.osid.id.Id sourceNodeId, 
                                                             org.osid.id.Id destinationNodeId, 
                                                             org.osid.calendaring.DateTime from, 
                                                             org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an EdgeList corresponding to the given peer <code> Ids </code> 
     *  and edge genus type and includes and genus types derived from the 
     *  given genus type. <code> </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all of the 
     *  edges corresponding to the given peer, including duplicates, or an 
     *  error results if an edge is inaccessible. Otherwise, inaccessible 
     *  <code> Edges </code> may be omitted from the list and may present the 
     *  elements in any order including returning a unique set. 
     *  
     *  In effective mode, edges are returned that are currently effective. In 
     *  any effective mode, effective edges and those currently expired are 
     *  returned. 
     *
     *  @param  sourceNodeId a node <code> Id </code> 
     *  @param  destinationNodeId a node <code> Id </code> 
     *  @param  edgeGenusType an edge genus type 
     *  @return the edges 
     *  @throws org.osid.NullArgumentException <code> sourceNodeId, 
     *          destinationNodeId, </code> or <code> edgeGenusType </code> is 
     *          <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.topology.EdgeList getEdgesByGenusTypeForNodes(org.osid.id.Id sourceNodeId, 
                                                                  org.osid.id.Id destinationNodeId, 
                                                                  org.osid.type.Type edgeGenusType)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets an <code> EdgeList </code> of the given genus type effective 
     *  during the entire given date range inclusive but not confined to the 
     *  date range. <code> </code> 
     *  
     *  <code> </code> In plenary mode, the returned list contains all known 
     *  edges or an error results. Otherwise, the returned list may contain 
     *  only those edges that are accessible through this session. 
     *  
     *  In effective mode, edges are returned that are currently effective. In 
     *  any effective mode, effective edges and those currently expired are 
     *  returned. 
     *
     *  @param  sourceNodeId a node <code> Id </code> 
     *  @param  destinationNodeId a node <code> Id </code> 
     *  @param  edgeGenusType an edge genus type 
     *  @param  from starting date 
     *  @param  to ending date 
     *  @return a list of edges 
     *  @throws org.osid.InvalidArgumentException <code> from </code> is 
     *          greater than <code> to </code> 
     *  @throws org.osid.NullArgumentException <code> sourceNodeId, 
     *          destinationNodeId, edgeGenusType, from </code> or <code> to 
     *          </code> is <code> null </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.topology.EdgeList getEdgesByGenusTypeForNodesOnDate(org.osid.id.Id sourceNodeId, 
                                                                        org.osid.id.Id destinationNodeId, 
                                                                        org.osid.type.Type edgeGenusType, 
                                                                        org.osid.calendaring.DateTime from, 
                                                                        org.osid.calendaring.DateTime to)
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;


    /**
     *  Gets all <code> Edges. </code> 
     *  
     *  In plenary mode, the returned list contains all known edges or an 
     *  error results. Otherwise, the returned list may contain only those 
     *  edges that are accessible through this session. 
     *  
     *  In effective mode, edges are returned that are currently effective. In 
     *  any effective mode, effective edges and those currently expired are 
     *  returned. 
     *
     *  @return a list of <code> Edges </code> 
     *  @throws org.osid.OperationFailedException unable to complete request 
     *  @throws org.osid.PermissionDeniedException authorization failure 
     *  @compliance mandatory This method must be implemented. 
     */

    public org.osid.topology.EdgeList getEdges()
        throws org.osid.OperationFailedException,
               org.osid.PermissionDeniedException;
}
